/*
    XorGramana Copyright 2009 James W. Morris, james@jwm-art.net

    This file is part of XorGramana.

    XorGramana is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    XorGramana is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with XorGramana.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "actions.h"
#include "map.h"
#include "gfx.h"
#include "options.h"

#include <stdlib.h>

struct xor_action actions[ICON_XXX]=
{
  {/* FLOOR */      MVI_NONE,       CT_PASS,            MV_NONE,    MV_NONE     },
  {/* WALL */       MVI_NONE,       CT_BLOCK,           MV_NONE,    MV_NONE     },
  {/* H_FIELD */    MVI_NONE,       CT_FILTER,          MV_NONE,    MV_HORIZ    },
  {/* V_FIELD */    MVI_NONE,       CT_FILTER,          MV_NONE,    MV_VERT     },
  {/* PLAYER0 */    MVI_PLAYER,     CT_BLOCK,           MV_ANY,     MV_NONE     },
  {/* PLAYER1 */    MVI_PLAYER,     CT_BLOCK,           MV_ANY,     MV_NONE     },
  {/* MAP */        MVI_NONE,       CT_PICKUP,          MV_NONE,    MV_NONE     },
  {/* TELEPORT*/    MVI_NONE,       CT_TELEPORT,        MV_NONE,    MV_NONE     },
  {/* EXIT_CLOSED */MVI_NONE,       CT_BLOCK,           MV_NONE,    MV_NONE     },
  {/* MASK */       MVI_NONE,       CT_PICKUP,          MV_NONE,    MV_NONE     },
  {/* DOT */        MVI_NONE,       CT_BLOCK,           MV_NONE,    MV_NONE     },
  {/* BLANK */      MVI_NONE,       CT_BLOCK,           MV_NONE,    MV_NONE     },
  /* the following will have behavioural indicators overlaid upon them          */
  {/* A (!fish)*/   MVI_GRAVITY,    CT_PUSH,            MV_UP,      MV_HORIZ    },
  {/* B (flip-dpq)*/MVI_NONE,       CT_PUSH|CT_FLIP,    MV_NONE,    MV_ANY      },
  {/* C (STATIC)*/  MVI_NONE,       CT_BLOCK,           MV_NONE,    MV_NONE     },
  {/* D (flip-bpq)*/MVI_NONE,       CT_PUSH|CT_FLIP,    MV_NONE,    MV_ANY      },
  {/* E (v-doll)*/  MVI_NONE,       CT_HARDPUSH,        MV_NONE,    MV_VERT     },
  {/* F (h-doll)*/  MVI_NONE,       CT_HARDPUSH,        MV_NONE,    MV_HORIZ    },
  {/* G (doll)*/    MVI_NONE,       CT_HARDPUSH,        MV_NONE,    MV_ANY      },
  {/* H (h-bomb)*/  MVI_GRAVITY,    CT_PUSH|CT_EXPLODE, MV_DOWN,    MV_HORIZ    },
  {/* I (STATIC)*/  MVI_NONE,       CT_BLOCK,           MV_NONE,    MV_NONE     },
  {/* J (flip-j)*/  MVI_NONE,       CT_PUSH|CT_FLIP,    MV_NONE,    MV_HORIZ    },
  {/* K (chick)*/   MVI_GRAVITY,    CT_PUSH,            MV_LEFT,    MV_VERT     },
  {/* L (flip-j)*/  MVI_NONE,       CT_PUSH|CT_FLIP,    MV_NONE,    MV_VERT     },
  {/* M (eeek)*/    MVI_GRAVITY,    CT_HARDPUSH,        MV_LEFT,    MV_VERT     },
  {/* N (eeek)*/    MVI_GRAVITY,    CT_HARDPUSH,        MV_RIGHT,   MV_VERT     },
  {/* O (!h-bomb)*/ MVI_GRAVITY,    CT_PUSH|CT_EXPLODE, MV_UP,      MV_HORIZ    },
  {/* P (flip-bdq)*/MVI_NONE,       CT_PUSH|CT_FLIP,    MV_NONE,    MV_ANY      },
  {/* Q (flip-bdp)*/MVI_NONE,       CT_PUSH|CT_FLIP,    MV_NONE,    MV_ANY      },
  {/* R (eek)*/     MVI_GRAVITY,    CT_HARDPUSH,        MV_UP,      MV_HORIZ    },
  {/* S (flip-z)*/  MVI_NONE,       CT_PUSH|CT_FLIP,    MV_NONE,    MV_HORIZ    },
  {/* T (!v-bomb)*/ MVI_GRAVITY,    CT_PUSH|CT_EXPLODE, MV_RIGHT,   MV_VERT,    },
  {/* U (eek)*/     MVI_GRAVITY,    CT_HARDPUSH,        MV_DOWN,    MV_HORIZ    },
  {/* V (fish)*/    MVI_GRAVITY,    CT_PUSH,            MV_DOWN,    MV_HORIZ    },
  {/* W (!chick)*/  MVI_GRAVITY,    CT_PUSH,            MV_RIGHT,   MV_VERT     },
  {/* X (v-bomb)*/  MVI_GRAVITY,    CT_PUSH|CT_EXPLODE, MV_LEFT,    MV_VERT     },
  {/* Y (switch)*/  MVI_NONE,       CT_PICKUP|CT_SWITCH,MV_NONE,    MV_NONE     },
  {/* Z (flip-s)*/  MVI_NONE,       CT_PUSH|CT_FLIP,    MV_NONE,    MV_VERT     },
  {/* 0 */          MVI_NONE, 0,0,0}, 
  {/* 1 */          MVI_NONE, 0,0,0}, /* for these numerical icons no behaviour */
  {/* 2 */          MVI_NONE, 0,0,0}, /* is yet defined - but will be sooner or */
  {/* 3 */          MVI_NONE, 0,0,0}, /* later..                                */
  {/* 4 */          MVI_NONE, 0,0,0},
  {/* 5 */          MVI_NONE, 0,0,0},
  {/* 6 */          MVI_NONE, 0,0,0},
  {/* 7 */          MVI_NONE, 0,0,0},
  {/* 8 */          MVI_NONE, 0,0,0},
  {/* 9 */          MVI_NONE, 0,0,0},
  /* the open exit cannot be specified in map file */
  {/* OPEN_EXIT */  MVI_NONE,       CT_EXIT,            MV_NONE,    MV_NONE     },
  /* the following are not ever appearing in map so have no defined behaviour   */
  {/*space trans*/  0,0,0,0},
  {/*space opaque*/ 0,0,0,0},
  {/*explosion1*/   0,0,0,0},
  {/*explosion2*/   0,0,0,0},
  {/*explosion3*/   0,0,0,0},
  {/*explosion4*/   0,0,0,0},
  {/*explosion5*/   0,0,0,0},
  {/*line_l*/       0,0,0,0},
  {/*line_r*/       0,0,0,0},
  {/*line_t*/       0,0,0,0},
  {/*line_b*/       0,0,0,0},
  {/*line_tl*/      0,0,0,0},
  {/*line_tr*/      0,0,0,0},
  {/*line_bl*/      0,0,0,0},
  {/*line_br*/      0,0,0,0},
  {/*loaded_xxx (MARKER) */ 0,0,0,0},
  /* the following are generated wall icons with edging, (set to behave as wall)*/
  {/*edge_t*/       MVI_NONE,       CT_BLOCK,           MV_NONE,    MV_NONE     },
  {/*edge_b*/       MVI_NONE,       CT_BLOCK,           MV_NONE,    MV_NONE     },
  {/*edge_l*/       MVI_NONE,       CT_BLOCK,           MV_NONE,    MV_NONE     },
  {/*edge_r*/       MVI_NONE,       CT_BLOCK,           MV_NONE,    MV_NONE     },
  {/*edge_tb*/      MVI_NONE,       CT_BLOCK,           MV_NONE,    MV_NONE     },
  {/*edge_lr*/      MVI_NONE,       CT_BLOCK,           MV_NONE,    MV_NONE     },
  {/*edge_tl*/      MVI_NONE,       CT_BLOCK,           MV_NONE,    MV_NONE     },
  {/*edge_tr*/      MVI_NONE,       CT_BLOCK,           MV_NONE,    MV_NONE     },
  {/*edge_bl*/      MVI_NONE,       CT_BLOCK,           MV_NONE,    MV_NONE     },
  {/*edge_br*/      MVI_NONE,       CT_BLOCK,           MV_NONE,    MV_NONE     },
  {/*edge_tbl*/     MVI_NONE,       CT_BLOCK,           MV_NONE,    MV_NONE     },
  {/*edge_tbr*/     MVI_NONE,       CT_BLOCK,           MV_NONE,    MV_NONE     },
  {/*edge_tlr*/     MVI_NONE,       CT_BLOCK,           MV_NONE,    MV_NONE     },
  {/*edge_blr*/     MVI_NONE,       CT_BLOCK,           MV_NONE,    MV_NONE     },
  {/*edge_tblr*/    MVI_NONE,       CT_BLOCK,           MV_NONE,    MV_NONE     }
};


void icons_init_overlays()
{
    int i;
    int o;
    for(i=ICON_A;i<=ICON_9;i++){
        #ifdef ICON_DEBUG
        printf("generating icon overlay for %d(%s)\n",i,icon_details[i].fname);
        #endif
        if(actions[i].mvini==MVI_GRAVITY){
            icons_with_indicators[i]=icon_duplicate(icons[i],TRUE,0,0,0);
            switch(actions[i].mvi_dir){
                case MV_LEFT:   o=DIRIND_GRAV_L;    break;
                case MV_RIGHT:  o=DIRIND_GRAV_R;    break;
                case MV_UP:     o=DIRIND_GRAV_U;    break;
                case MV_DOWN:   o=DIRIND_GRAV_D;    break;
            }
            overlay_image(icons_with_indicators[i],indicators[o],0,0);
        }
        if(actions[i].cont&CT_PUSH){
            if(!icons_with_indicators[i])
                icons_with_indicators[i]=icon_duplicate(icons[i],TRUE,0,0,0);
            if(actions[i].cont_dir&MV_LEFT)
                overlay_image(icons_with_indicators[i],indicators[DIRIND_PUSH_L],0,0);
            if(actions[i].cont_dir&MV_RIGHT)
                overlay_image(icons_with_indicators[i],indicators[DIRIND_PUSH_R],0,0);
            if(actions[i].cont_dir&MV_UP)
                overlay_image(icons_with_indicators[i],indicators[DIRIND_PUSH_U],0,0);
            if(actions[i].cont_dir&MV_DOWN)
                overlay_image(icons_with_indicators[i],indicators[DIRIND_PUSH_D],0,0);
        }
        if(actions[i].cont&CT_HARDPUSH){
            if(!icons_with_indicators[i])
                icons_with_indicators[i]=icon_duplicate(icons[i],TRUE,0,0,0);
            if(actions[i].cont_dir&MV_LEFT)
                overlay_image(icons_with_indicators[i],indicators[DIRIND_HARDPUSH_L],0,0);
            if(actions[i].cont_dir&MV_RIGHT)
                overlay_image(icons_with_indicators[i],indicators[DIRIND_HARDPUSH_R],0,0);
            if(actions[i].cont_dir&MV_UP)
                overlay_image(icons_with_indicators[i],indicators[DIRIND_HARDPUSH_U],0,0);
            if(actions[i].cont_dir&MV_DOWN)
                overlay_image(icons_with_indicators[i],indicators[DIRIND_HARDPUSH_D],0,0);
        }
        if(actions[i].cont&CT_EXPLODE){
            if(!icons_with_indicators[i])
                icons_with_indicators[i]=icon_duplicate(icons[i],TRUE,0,0,0);
            if(actions[i].cont_dir&MV_HORIZ)
                overlay_image(icons_with_indicators[i],indicators[DIRIND_EXPLODE_H],0,0);
            if(actions[i].cont_dir&MV_VERT)
                overlay_image(icons_with_indicators[i],indicators[DIRIND_EXPLODE_V],0,0);
        }
    }
}

struct xor_move* create_xor_move(xy_t x, xy_t y, su_t move)
{
    struct xor_move* xmv;
    if(!(xmv=malloc(sizeof(struct xor_move))))
        return 0;
    xmv->from_obj=map->data[y][x];
    xmv->from_x=x;
    xmv->from_y=y;
    xmv->dir=move;
    xmv->to_x=xmv->to_y=0;
    xmv->up=xmv->down=xmv->right=xmv->left=0;
    xmv->moves_count=0;
    return xmv;
}

/*
    the following two functions are a bit cyclical in their
    operation so think twice if you spot something seeming
    at first glance nonsensical. (ie james morris, you/me,
    we know we have a habbit of scanning our code and
    thinking wtf!?!? and then editing without realising...
    and we hate these comments which now seem obvious and
    so untidy...
*/
#include "player.h"

struct xor_move* create_gravity_chain_xydir(xy_t x, xy_t y, su_t dir)
{
    struct xor_move* head=0;
    struct xor_move* xmv=0;
    struct xor_move* tmp=0;
    int icon;
    do{
        icon=map->data[y][x];
        if(actions[icon].mvini!=MVI_GRAVITY)
            return head;
        if(actions[icon].mvi_dir!=dir)
            return head;
        if(!(xmv=malloc(sizeof(struct xor_move))))
            return head;
        if(!head)
            head=xmv;
        if(tmp){
            switch(dir){
                case MV_UP:     tmp->down=xmv;  break;
                case MV_DOWN:   tmp->up=xmv;    break;
                case MV_LEFT:   tmp->right=xmv; break;
                case MV_RIGHT:  tmp->left=xmv;  break;
            }
        }
        xmv->from_obj=icon;
        xmv->from_x=x;
        xmv->from_y=y;
        xmv->dir=dir;
        xmv->to_x=xmv->to_y=0;
        xmv->up=xmv->down=xmv->right=xmv->left=0;
        xmv->moves_count=0;
        tmp=xmv;
        switch(dir){
            case MV_UP:     y++;    break;
            case MV_DOWN:   y--;    break;
            case MV_LEFT:   x++;    break;
            case MV_RIGHT:  x--;    break;
        }
    }while(TRUE);
    return head;
}

struct xor_move* create_gravity_chain(struct xor_move* xmv)
{
    struct xor_move* head=0;
    struct xor_move* tmp=0;
    struct xor_move* oxmv=xmv;
    su_t icon;
    xy_t x, y;
    su_t dir;
    x=xmv->from_x;
    y=xmv->from_y;
    dir=xmv->dir;
    do{
        icon=map->data[y][x];
        if(actions[icon].mvini!=MVI_GRAVITY){
            if(xmv!=oxmv)
                free(xmv);
            return head;
        }
        if(actions[icon].mvi_dir!=dir){
            if(xmv!=oxmv)
                free(xmv);
            return head;
        }
        if(!head)
            head=xmv;
        if(tmp){
            switch(dir){
                case MV_UP:     tmp->down=xmv;  break;
                case MV_DOWN:   tmp->up=xmv;    break;
                case MV_LEFT:   tmp->right=xmv; break;
                case MV_RIGHT:  tmp->left=xmv;  break;
            }
        }
        xmv->from_obj=icon; /* these are redundant */
        xmv->from_x=x;      /* on the first pass   */
        xmv->from_y=y;      /* oh well...          */
        xmv->dir=dir;
        xmv->to_x=xmv->to_y=0;
        xmv->up=xmv->down=xmv->right=xmv->left=0;
        xmv->moves_count=0;
        tmp=xmv;
        switch(dir){
            case MV_UP:     y++;    break;
            case MV_DOWN:   y--;    break;
            case MV_LEFT:   x++;    break;
            case MV_RIGHT:  x--;    break;
        }
        if(!(xmv=malloc(sizeof(struct xor_move))))
            return head;
    }while(TRUE);
    return head;
}

void destroy_gravity_chain(struct xor_move* xmv)
{
    struct xor_move* tmp;
    while(xmv){
        tmp=xmv;
        switch(xmv->dir){
            case MV_UP:     xmv=xmv->down;  break;
            case MV_DOWN:   xmv=xmv->up;    break;
            case MV_LEFT:   xmv=xmv->right; break;
            case MV_RIGHT:  xmv=xmv->left;  break;
        }
        free(tmp);
    }
}

su_t bomb_type(su_t icon)
{
    if(actions[icon].cont&CT_EXPLODE)
        return actions[icon].cont_dir;
    return MV_NONE;
}
