/*
    XorGramana Copyright 2009 James W. Morris, james@jwm-art.net

    This file is part of XorGramana.

    XorGramana is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    XorGramana is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with XorGramana.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _ACTIONS_H
#define _ACTIONS_H

#include "types.h"
#include "icons.h"
/*
    contact here means one object (in movement) attempting to
    move onto another object - ie two objects are not in
    contact when situated side by side.
*/

enum MOVES
{
    /* movement directions */
    MV_NONE=    0x00,
    MV_LEFT=    0x01,
    MV_RIGHT=   0x02,
    MV_UP=      0x04,
    MV_DOWN=    0x08,

    /* combos - happenings */
    MV_HORIZ=   0x03,
    MV_VERT=    0x0c,
    MV_ANY=     0x0f, /* ALL/ANY (direction (context)) */

    /* special extra-ordinary cases (not used by routines here) */
    MV_PLAYER_SWAP=0x10,
    MV_PLAYER_QUIT=0x20
};

enum MVINIT         /* movement without contact */
{
    MVI_NONE=       0,  /* does not move (but might be pushed)      */
    MVI_PLAYER=     1,  /* moved by player hitting cursor keys ;-)  */
    MVI_GRAVITY=    2   /* moved by gravity (when path unblocked)   */
};

enum CONTACT        /* happenings initiated by contact */
{
    CT_BLOCK=       0x000,  /* blocks movement in all directions        */
    CT_PASS=        0x001,  /* object does nothing (ie space)           */
    CT_FILTER=      0x002,  /* object filters movement                  */
    CT_PICKUP=      0x004,  /* object is collected (player)             */
    CT_PUSH=        0x008,  /* object is pushed (player)                */
    CT_FLIP=        0x010,  /* object flips                             */
    CT_HARDPUSH=    0x020,  /* object keeps moving (needs momentum)     */
    CT_EXPLODE=     0x040,  /* object explodes (non player contact)     */
    CT_TELEPORT=    0x080,  /* object teleports player                  */
    CT_SWITCH=      0x100,  /* toggle wall visibility                   */
    CT_EXIT=        0x200,  /* easy coded exit level if masks collected */
    CT_ERROR=       0x400
};

struct xor_action
{
    unsigned mvini    : 4;
    unsigned cont     : 12;
    unsigned mvi_dir  : 4;
    unsigned cont_dir : 4;
};

struct xor_move /* one per object determined to have potential movement */
{
    xy_t from_x;
    xy_t from_y;
    su_t from_obj;
    su_t dir;
    xy_t to_x;
    xy_t to_y;
    su_t to_obj;
    su_t moves_count;    /* count of moves (gravitating objects only)*/
    /* links for chains of xor moves */
    struct xor_move* up;
    struct xor_move* down;
    struct xor_move* right;
    struct xor_move* left;
};

extern struct xor_action actions[ICON_XXX];

/*  perhaps icons_init_overlays should be in icons.h+icons.c,   *
 *  but seeing as it requires the above array to be initialised *
 *  that is, what this func does depends on that array, it is   *
 *  in here, so as to avoid premature calling (???)             */

void icons_init_overlays();

/*
    create_xor_move:
        to move a known object in specified direction
*/
struct xor_move* create_xor_move(xy_t x, xy_t y, su_t move);

/*
    these two functions create a chain of xor_moves where
    the chain begins at map x,y and continues in the opposite
    direction to dir, where each following xor_move holds the
    same characteristics. the chain ends when an object in the
    map is encountered which does not have the relevant
    characteristics. returns the head of the chain or 0 if
    no object with the relevant characteristics is found.
    second form takes x,y,dir from the xor_move passed.
*/
struct xor_move* create_gravity_chain_xydir(xy_t x, xy_t y, su_t dir);
struct xor_move* create_gravity_chain(struct xor_move* xmv);
void destroy_gravity_chain(struct xor_move* xmv);

su_t bomb_type(su_t icon);

#endif
