/*
    XorGramana Copyright 2009 James W. Morris, james@jwm-art.net

    This file is part of XorGramana.

    XorGramana is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    XorGramana is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with XorGramana.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "explode.h"

#include "game_display.h"

#include "player.h"
#include "options.h"

#include <stdlib.h>

void explode_process_detonator(struct xmv_link* detlnk)
{
    struct xor_move* detmv=detlnk->xmv;
    xy_t bombx=detmv->to_x;
    xy_t bomby=detmv->to_y;
    xy_t exp_x[4]={0,0,0,0};
    xy_t exp_y[4]={0,0,0,0};
    enum MOVES detdir=detmv->dir;
    su_t i;
    su_t icon;
    xy_t xi;
    su_t dead1=0;
    su_t dead2=0;
    su_t exp_dir;
    bool exp[4]={TRUE,TRUE,TRUE,TRUE};
    unsigned int flimit=SDL_GetTicks()+TICK_COUNT;
/*
    rpause.tv_sec=0;
    if(player.replay)
        rpause.tv_nsec=options_replay_speed(options->replay_speed);
    else
        rpause.tv_nsec=60000000L;
*/
    exp_x[0]=detmv->from_x;
    exp_y[0]=detmv->from_y;
    exp_x[1]=bombx;
    exp_y[1]=bomby;
    switch((exp_dir=actions[detmv->to_obj].cont_dir)){
        case MV_HORIZ:  /* type is h-bomb */
            exp_x[2]=bombx-1;
            exp_y[2]=bomby;
            if(detdir==MV_LEFT)
                exp[3]=FALSE;
            else{
                exp_x[3]=bombx+1;
                exp_y[3]=bomby;
            }
            break;
        case MV_VERT:   /* type is v-bomb */
            if(detdir==MV_DOWN)
                exp[2]=FALSE;
            else{
                exp_x[2]=bombx;
                exp_y[2]=bomby-1;
            }
            exp_x[3]=bombx;
            exp_y[3]=bomby+1;
            break;
        default:
            fprintf(stderr,"Bomb has invalid explosion type!\n");
            fprintf(stderr,"Aborting detonation!\n");
            fprintf(stderr,"*** Movement left in abandonded state:LEAK CHECK");
            return;
    }
    for(i=0;i<4;i++)
        if(exp_x[i]==0||exp_x[i]>map->width-2
         ||exp_y[i]==0||exp_y[i]>map->height-2)
            exp[i]=FALSE;
    for(xi=ICON_EXPLOSION1;xi<=ICON_EXPLOSION5;xi++){
        for(i=0;i<4;i++){
            if(exp[i])
                game_win_icon_display(exp_x[i],exp_y[i],xi);
        }
        /*  wrefresh(game_win); */
        SDL_Flip(screen);
        delay(flimit);
        flimit=SDL_GetTicks()+TICK_COUNT;
    }
    for(i=0;i<4;i++)
        if(exp[i]){
            game_win_icon_display(exp_x[i],exp_y[i],ICON_FLOOR);
            icon=map->data[exp_y[i]][exp_x[i]];
            if(actions[icon].mvini==MVI_PLAYER){
                if(dead1==0)
                    dead1=icon;
                else
                    dead2=icon;
            }
            else if(actions[icon].cont&CT_SWITCH){
                player.wall_vis=(player.wall_vis?FALSE:TRUE);
                init_wall(map->level,player.wall_vis);
            }
            map->data[exp_y[i]][exp_x[i]]=ICON_FLOOR;
            if(actions[icon].cont&CT_SWITCH)
                game_win_display();
        }
    if(dead1)
        player_death(dead1);
    if(dead2)
        player_death(dead2);
    /*  if detonator part of chain, remove it from chain, delete it,
        and set detlnk to point to new head of chain. if not part
        of a chain, unlink, and delete.
    */

    if(detmv->right){
        detlnk->xmv=detmv->right;
        free(detmv);
        detmv=0;
    }
    else if(detmv->up){
        detlnk->xmv=detmv->up;
        free(detmv);
        detmv=0;
    }
    else if(detmv->left){
        detlnk->xmv=detmv->left;
        free(detmv);
        detmv=0;
    }
    else if(detmv->down){
        detlnk->xmv=detmv->down;
        free(detmv);
        detmv=0;
    }
    else
        free(xmvlist_unlink_xor_move(detlnk));
    switch(exp_dir){
        case MV_HORIZ:
            explode_process_h_blast(bombx,bomby,detdir);
            break;
        case MV_VERT:
            explode_process_v_blast(bombx,bomby,detdir);
            break;
    }
}

void explode_process_h_blast(xy_t bombx, xy_t bomby, su_t detdir)
{
    struct xor_move* xmv;
    xy_t chkx[MAX_EXP_CHECKS];
    xy_t chky[MAX_EXP_CHECKS];
    xy_t dir[MAX_EXP_CHECKS];
    su_t i;
    su_t max_chk;
    switch(detdir){
        case MV_DOWN:
            chkx[0]=bombx+1;    chky[0]=bomby-1;    dir[0]=MV_DOWN;
            chkx[1]=bombx+1;    chky[1]=bomby-1;    dir[1]=MV_LEFT;
            chkx[2]=bombx+1;    chky[2]=bomby+1;    dir[2]=MV_UP;
            chkx[3]=bombx;      chky[3]=bomby+1;    dir[3]=MV_UP;
            chkx[4]=bombx-1;    chky[4]=bomby-1;    dir[4]=MV_DOWN;
            chkx[5]=bombx-1;    chky[5]=bomby-1;    dir[5]=MV_RIGHT;
            chkx[6]=bombx-1;    chky[6]=bomby+1;    dir[6]=MV_UP;
            chkx[7]=bombx+2;    chky[7]=bomby;      dir[7]=MV_LEFT;
            chkx[8]=bombx-2;    chky[8]=bomby;      dir[8]=MV_RIGHT;
            max_chk=9;
            break;
        case MV_LEFT:
            chkx[0]=bombx+1;    chky[0]=bomby-1;    dir[0]=MV_DOWN;
            chkx[1]=bombx+1;    chky[1]=bomby+1;    dir[1]=MV_UP;
            chkx[2]=bombx;      chky[2]=bomby-1;    dir[2]=MV_DOWN;
            chkx[3]=bombx;      chky[3]=bomby+1;    dir[3]=MV_UP;
            chkx[4]=bombx-1;    chky[4]=bomby-1;    dir[4]=MV_DOWN;
            chkx[5]=bombx-1;    chky[5]=bomby+1;    dir[5]=MV_UP;
            chkx[6]=bombx-2;    chky[6]=bomby;      dir[6]=MV_RIGHT;
            max_chk=7;
            break;
        case MV_UP:
            chkx[0]=bombx+1;    chky[0]=bomby-1;    dir[0]=MV_DOWN;
            chkx[1]=bombx+1;    chky[1]=bomby+1;    dir[1]=MV_UP;
            chkx[2]=bombx+1;    chky[2]=bomby+1;    dir[2]=MV_LEFT;
            chkx[3]=bombx;      chky[3]=bomby-1;    dir[3]=MV_DOWN;
            chkx[4]=bombx-1;    chky[4]=bomby-1;    dir[4]=MV_DOWN;
            chkx[5]=bombx-1;    chky[5]=bomby+1;    dir[5]=MV_UP;
            chkx[6]=bombx-1;    chky[6]=bomby+1;    dir[6]=MV_RIGHT;
            chkx[7]=bombx+2;    chky[7]=bomby;      dir[7]=MV_LEFT;
            chkx[8]=bombx-2;    chky[8]=bomby;      dir[8]=MV_RIGHT;
            max_chk=9;
            break;
        case MV_RIGHT:
            chkx[0]=bombx+1;    chky[0]=bomby-1;    dir[0]=MV_DOWN;
            chkx[1]=bombx+1;    chky[1]=bomby+1;    dir[1]=MV_UP;
            chkx[2]=bombx;      chky[2]=bomby-1;    dir[2]=MV_DOWN;
            chkx[3]=bombx;      chky[3]=bomby+1;    dir[3]=MV_UP;
            chkx[4]=bombx-1;    chky[4]=bomby-1;    dir[4]=MV_DOWN;
            chkx[5]=bombx-1;    chky[5]=bomby+1;    dir[5]=MV_UP;
            chkx[6]=bombx+2;    chky[6]=bomby;      dir[6]=MV_LEFT;
            max_chk=7;
            break;
    }
    #ifdef EXP_DEBUG
    for(i=0;i<max_chk;i++){
        printf("check:%d x:%d,y:%d ",i,chkx[i],chky[i]);
        switch(dir[i]){
            case MV_LEFT:printf("left ");break;
            case MV_RIGHT:printf("right ");break;
            case MV_UP:printf("up ");break;
            case MV_DOWN:printf("down");break;
        }
        printf(" map item: %s\n", icon_details[map->data[chky[i]][chkx[i]]].fname);
    }
    #endif
    for(i=0;i<max_chk;i++)
        if(chkx[i]>0&&chkx[i]<map->width-1&&chky[i]>0&&chky[i]<map->height-1)
            if(!xmvlist_contains_coord(chkx[i],chky[i],0,0))
                if((xmv=create_gravity_chain_xydir(chkx[i],chky[i],dir[i])))
                    xmvlist_append_xor_move(xmv);
}

void explode_process_v_blast(xy_t bombx, xy_t bomby, su_t detdir)
{
    struct xor_move* xmv;
    xy_t chkx[MAX_EXP_CHECKS];
    xy_t chky[MAX_EXP_CHECKS];
    su_t dir[MAX_EXP_CHECKS];
    su_t i;
    su_t max_chk;
    #ifdef EXP_DEBUG
    printf("explode_process_v_blast(bombx:%d, bomby:%d, detdir:%d)\n",
        bombx,bomby,detdir);
    #endif
    switch(detdir){
        case MV_DOWN:
            chkx[0]=bombx+1;  chky[0]=bomby-1;  dir[0]=MV_LEFT;
            chkx[1]=bombx-1;  chky[1]=bomby-1;  dir[1]=MV_RIGHT;
            chkx[2]=bombx+1;  chky[2]=bomby;    dir[2]=MV_LEFT;
            chkx[3]=bombx-1;  chky[3]=bomby;    dir[3]=MV_RIGHT;
            chkx[4]=bombx+1;  chky[4]=bomby+1;  dir[4]=MV_LEFT;
            chkx[5]=bombx-1;  chky[5]=bomby+1;  dir[5]=MV_RIGHT;
            chkx[6]=bombx;    chky[6]=bomby+2;  dir[6]=MV_UP;
            max_chk=7;
            break;
        case MV_LEFT:
            chkx[0]=bombx+1;  chky[0]=bomby-1;  dir[0]=MV_LEFT;
            chkx[1]=bombx+1;  chky[1]=bomby-1;  dir[1]=MV_DOWN;
            chkx[2]=bombx-1;  chky[2]=bomby-1;  dir[2]=MV_RIGHT;
            chkx[3]=bombx-1;  chky[3]=bomby;    dir[3]=MV_RIGHT;
            chkx[4]=bombx+1;  chky[4]=bomby+1;  dir[4]=MV_LEFT;
            chkx[5]=bombx+1;  chky[5]=bomby+1;  dir[5]=MV_UP;
            chkx[6]=bombx-1;  chky[6]=bomby+1;  dir[6]=MV_RIGHT;
            chkx[7]=bombx;    chky[7]=bomby-2;  dir[7]=MV_DOWN;
            chkx[8]=bombx;    chky[8]=bomby+2;  dir[8]=MV_UP;
            max_chk=9;
            break;
        case MV_UP:
            chkx[0]=bombx+1;  chky[0]=bomby-1;  dir[0]=MV_LEFT;
            chkx[1]=bombx-1;  chky[1]=bomby-1;  dir[1]=MV_RIGHT;
            chkx[2]=bombx+1;  chky[2]=bomby;    dir[2]=MV_LEFT;
            chkx[3]=bombx-1;  chky[3]=bomby;    dir[3]=MV_RIGHT;
            chkx[4]=bombx+1;  chky[4]=bomby+1;  dir[4]=MV_LEFT;
            chkx[5]=bombx-1;  chky[5]=bomby+1;  dir[5]=MV_RIGHT;
            chkx[6]=bombx;    chky[6]=bomby-2;  dir[6]=MV_DOWN;
            max_chk=7;
            break;
        case MV_RIGHT:
            chkx[0]=bombx+1;  chky[0]=bomby-1;  dir[0]=MV_LEFT;
            chkx[1]=bombx-1;  chky[1]=bomby-1;  dir[1]=MV_RIGHT;
            chkx[2]=bombx-1;  chky[2]=bomby-1;  dir[2]=MV_DOWN;
            chkx[3]=bombx+1;  chky[3]=bomby;    dir[3]=MV_LEFT;
            chkx[4]=bombx+1;  chky[4]=bomby+1;  dir[4]=MV_LEFT;
            chkx[5]=bombx-1;  chky[5]=bomby+1;  dir[5]=MV_RIGHT;
            chkx[6]=bombx-1;  chky[6]=bomby+1;  dir[6]=MV_UP;
            chkx[7]=bombx;    chky[7]=bomby-2;  dir[7]=MV_DOWN;
            chkx[8]=bombx;    chky[8]=bomby+2;  dir[8]=MV_UP;
            max_chk=9;
            break;
    }
    #ifdef EXP_DEBUG
    for(i=0;i<max_chk;i++){
        printf("check:%d x:%d,y:%d ",i,chkx[i],chky[i]);
        switch(dir[i]){
            case MV_LEFT:printf("left ");break;
            case MV_RIGHT:printf("right ");break;
            case MV_UP:printf("up ");break;
            case MV_DOWN:printf("down");break;
        }
        printf(" map item: %s\n", icon_details[map->data[chky[i]][chkx[i]]].fname);
    }
    #endif
    for(i=0;i<max_chk;i++)
        if(chkx[i]>0&&chkx[i]<map->width-1&&chky[i]>0&&chky[i]<map->height-1)
            if(!xmvlist_contains_coord(chkx[i],chky[i],0,0))
                if((xmv=create_gravity_chain_xydir(chkx[i],chky[i],dir[i])))
                    xmvlist_append_xor_move(xmv);
}
