/*
    XorGramana Copyright 2009 James W. Morris, james@jwm-art.net

    This file is part of XorGramana.

    XorGramana is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    XorGramana is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with XorGramana.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "gfx.h"
#include "input.h"

#include <stdlib.h>
#include <stdarg.h>

SDL_Surface* screen=0;
struct gfx_mode kermode;
struct gfx_win* scr_win=0;
struct gfx_win* main_win=0;
struct gfx_win* cur_win=0;

void gfx_start()
{
    int i,n;
    int rx,by;
    int mc=0;
    SDL_Rect **modes;
    if(SDL_Init(SDL_INIT_VIDEO)<0){
        fprintf(stderr,"Couldn't initialize SDL: %s\n",SDL_GetError());
        exit(-1);
    }
    SDL_WM_SetCaption("XorGramana",0);
    kermode.def=kermode.last=kermode.mode=-1;
    kermode.m=0;
    kermode.modes=0;
    modes=SDL_ListModes(NULL, SDL_FULLSCREEN|SDL_HWSURFACE);
    if(!modes){
        fprintf(stderr,"No modes available!\n");
        SDL_Quit();
        exit(-1);
    }
    if(modes == (SDL_Rect **)-1){
      fprintf(stderr,"All resolutions available.\n");
      fprintf(stderr,"Dont't know what to do!!!!\n Probably will crash!\n");
      /* now what? how do we know what they are? */
    }
    else{
        mc=0;
        for(i=0;modes[i];++i){
            #ifdef GFX_DEBUG
            printf("Checking mode: %d x %d ... ",
                modes[i]->w,
                modes[i]->h);
            #endif
            if(modes[i]->w<GFX_MIN_WIDTH
             ||modes[i]->h<GFX_MIN_HEIGHT)
            {
                modes[i]=0;
                #ifdef GFX_DEBUG
                printf("Too small.\n");
                #endif
            }
            else{
                #ifdef GFX_DEBUG
                printf("Ok.\n");
                #endif
                mc++;
            }
        }
    }
    #ifdef GFX_DEBUG
    printf("%d gfx modes found\n",mc);
    #endif
    if(!(kermode.modes=malloc(sizeof(SDL_Rect*)*mc))){
        fprintf(stderr,"Failed to allocate for gfx modes dit dit dit...");
        SDL_Quit();
        exit(-1);
    }
    if(mc==0){
        fprintf(stderr,
            "%d x %d video mode or larger not found.\n",
            GFX_MIN_WIDTH,GFX_MIN_HEIGHT);
        fprintf(stderr,"Sorry, this is a minimum requirement.\n");
        SDL_Quit();
        exit(-1);
    }
    #ifdef GFX_LARGEST_RES
    kermode.mode=kermode.last=0;
    #else
    kermode.mode=kermode.last=mc-1;
    #endif
    mc=0;
    n=i;
    for(i=0;i<n;i++)
        if(modes[i]){
            kermode.modes[mc]=modes[i];
            mc++;
        }
    kermode.m=kermode.modes[kermode.mode];
    screen=SDL_SetVideoMode(kermode.m->w,kermode.m->h,0,
        SDL_HWSURFACE|SDL_HWPALETTE|SDL_DOUBLEBUF);
    if(!screen){
        fprintf(stderr,
            "Couldn't set %d x %d hardware surface video mode: %s\n",
            kermode.m->w,kermode.m->h,SDL_GetError());
        SDL_Quit();
        exit(-1);
    }
    scr_win=gfx_new_win(GFX_CENTER_X|GFX_CENTER_Y,0,0,0,0,ICON_WIDTH,ICON_HEIGHT);
    rx=scr_win->win_w-ICON_WIDTH;
    by=scr_win->win_h-ICON_HEIGHT;
    main_win=gfx_new_win(GFX_CENTER_X|GFX_CENTER_Y,0,0,rx,by,ICON_WIDTH,ICON_HEIGHT);
}

void gfx_end()
{
    free(scr_win);
    free(main_win);
    scr_win=0;
    SDL_Quit();
}

void scr_win_border()
{
    gfx_set_win(scr_win);
    gfx_win_half_size();
    gfx_box(0,0,scr_win->blk_count_x-1,scr_win->blk_count_y-1,FALSE);
}

void gfx_inc_mode()
{
}

void gfx_dec_mode()
{
}

void draw_image(SDL_Surface *image, int x, int y)
{
    SDL_Rect dest;
    dest.x=x;
    dest.y=y;
    dest.w=image->w;
    dest.h=image->h;
    SDL_BlitSurface(image, NULL, screen, &dest);
}

void gfx_win_draw_image(SDL_Surface *image, int blk_x, int blk_y)
{
    SDL_Rect dest;
    dest.x=cur_win->blk_tlx+blk_x*image->w;
    dest.y=cur_win->blk_tly+blk_y*image->h;
    dest.w=image->w;
    dest.h=image->h;
    SDL_BlitSurface(image, NULL, screen, &dest);
}

void overlay_image(SDL_Surface* image, SDL_Surface *overlay, int x, int y)
{
    SDL_Rect dest;
    dest.x=x;
    dest.y=y;
    dest.w=image->w;
    dest.h=image->h;
    SDL_BlitSurface(overlay, NULL, image, &dest);
}

Uint32 getpixel(SDL_Surface *surface, int x, int y)
{
    int bpp=surface->format->BytesPerPixel;
    /* Here p is the address to the pixel we want to retrieve */
    Uint8* p=(Uint8*)surface->pixels+y*surface->pitch+x*bpp;
    switch(bpp){
        case 1: return *p;
        case 2: return *(Uint16 *)p;
        case 3:
            if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
                return p[0] << 16 | p[1] << 8 | p[2];
            else
                return p[0] | p[1] << 8 | p[2] << 16;
        case 4: return *(Uint32 *)p;
        default:
            return 0;
    }
}

void putpixel(SDL_Surface *surface, int x, int y, Uint32 pixel)
{
    int bpp=surface->format->BytesPerPixel;
    /* Here p is the address to the pixel we want to set */
    Uint8* p=(Uint8*)surface->pixels+y*surface->pitch+x*bpp;
    switch(bpp){
        case 1: *p = pixel;             break;
        case 2: *(Uint16 *)p = pixel;   break;
        case 3: 
            if(SDL_BYTEORDER == SDL_BIG_ENDIAN) {
                p[0] = (pixel >> 16) & 0xff;
                p[1] = (pixel >> 8) & 0xff;
                p[2] = pixel & 0xff;
            }
            else{
                p[0] = pixel & 0xff;
                p[1] = (pixel >> 8) & 0xff;
                p[2] = (pixel >> 16) & 0xff;
            }
            break;
        case 4: *(Uint32 *)p = pixel;   break;
    }
}

struct gfx_win*
gfx_new_win(int flags,
            int tlx, int tly,int brx, int bry,
            int blk_width,int blk_height)
{
    struct gfx_win* win=0;
    int tmptlx,tmptly,tmpbrx,tmpbry;
    int offx=0;
    int offy=0;
    int tmp;
    if(tlx==0&&tly==0&&brx==0&&bry==0){
        brx=screen->w;
        bry=screen->h;
    }
    if(blk_width<2||blk_height<2)
        return 0;
    tmptlx=((tlx>=0?tlx:0)<screen->w?tlx:screen->w);
    tmptly=((tly>=0?tly:0)<screen->h?tly:screen->h);
    tmpbrx=((brx>=0?brx:0)<screen->w?brx:screen->w);
    tmpbry=((bry>=0?bry:0)<screen->h?bry:screen->h);
    if(tmptlx>=tmpbrx||tmptly>=tmpbry)
        return 0;
    if(!(win=malloc(sizeof(struct gfx_win))))
        return 0;
    win->win_w=tmpbrx-tmptlx;
    win->win_h=tmpbry-tmptly;
    if(flags&GFX_CENTER_X)
        offx=(screen->w - win->win_w)/2;
    else if(flags&GFX_ALIGN_R)
        offx=screen->w - win->win_w;
    if(flags&GFX_CENTER_Y)
        offy=(screen->h - win->win_h)/2;
    else if(flags&GFX_ALIGN_B)
        offy=screen->h - win->win_h;
    win->win_tlx=offx+tmptlx;
    win->win_tly=offy+tmptly;
    win->win_brx=offx+tmpbrx;
    win->win_bry=offy+tmpbry;
    if(flags&GFX_BLK_ALIGN_X){
        win->win_tlx=(win->win_tlx/blk_width)*blk_width;
        win->win_brx=(win->win_brx/blk_width)*blk_width;
        win->win_w=win->win_brx-win->win_tlx;
    }
    if(flags&GFX_BLK_ALIGN_Y){
        win->win_tly=(win->win_tly/blk_height)*blk_height;
        win->win_bry=(win->win_bry/blk_height)*blk_height;
        win->win_h=win->win_bry-win->win_tly;
    }
    win->win_cx=0;
    win->win_cy=0;

    /* calc block printing area */
    win->icon_width=blk_width;
    win->icon_height=blk_height;
    win->blk_count_x=win->win_w/blk_width;
    win->blk_count_y=win->win_h/blk_height;
    offx=0;
    offy=0;

    /* do the number of blocks fit exactly? */
    tmp=win->blk_count_x * blk_width;
    if(tmp<win->win_w)
        offx=(win->win_w-tmp)/2;
    tmp=win->blk_count_y * blk_height;
    if(tmp<win->win_h)
        offy=(win->win_h-tmp)/2;
    win->blk_tlx=win->win_tlx+offx;
    win->blk_brx=win->win_brx-offx;
    win->blk_tly=win->win_tly+offy;
    win->blk_bry=win->win_bry-offy;
    win->blk_cx=0;
    win->blk_cy=0;
    win->icons=icons;
    win->half_size=0;
    return win;
}

void gfx_set_win(struct gfx_win* win)
{
    cur_win=win;
}

void gfx_win_half_size()
{
    if(!cur_win){
        fprintf(stderr,"gfx_win_half_size: cur_win == NULL!\n");
        return;
    }
    if(cur_win->half_size)
        return;
    cur_win->half_size=1;
    cur_win->icons=hs_icons;
    cur_win->icon_width/=2;
    cur_win->icon_height/=2;
    cur_win->blk_cx*=2;
    cur_win->blk_cy*=2;
    cur_win->blk_count_x*=2;
    cur_win->blk_count_y*=2;
}

void gfx_win_full_size()
{
    if(!cur_win){
        fprintf(stderr,"gfx_win_full_size: cur_win == NULL!\n");
        return;
    }
    if(!cur_win->half_size)
        return;
    cur_win->half_size=0;
    cur_win->icons=icons;
    cur_win->icon_width*=2;
    cur_win->icon_height*=2;
    cur_win->blk_cx/=2;
    cur_win->blk_cy/=2;
    cur_win->blk_count_x/=2;
    cur_win->blk_count_y/=2;
}

su_t gfx_mv_cur(su_t blk_x, su_t blk_y)
{
    int x,y;
    if(!cur_win){
        fprintf(stderr,"gfx_mv_cur: cur_win == NULL!\n");
        return 0;
    }
    if(blk_x>cur_win->blk_count_x||blk_x<0
     ||blk_y>cur_win->blk_count_y||blk_y<0)
    {
        fprintf(stderr,
            "gfx_mv_cur: loc %d,%d outside range 0,0 - %d,%d\n",
            (int)blk_x, (int)blk_y,
            (int)cur_win->blk_count_x,(int)cur_win->blk_count_y);
        return 0;
    }
    x=cur_win->blk_tlx+blk_x*cur_win->icon_width;
    y=cur_win->blk_tly+blk_y*cur_win->icon_height;
    cur_win->blk_cx=blk_x;
    cur_win->blk_cy=blk_y;
    cur_win->win_cx=x;
    cur_win->win_cy=y;
    return 1;
}

void gfx_inc_cursor(bool line)
{
    if(!line){
        cur_win->blk_cx++;
        if(cur_win->blk_cx==cur_win->blk_count_x)
            line=TRUE;
    }
    if(line){
        cur_win->blk_cx=0;
        cur_win->blk_cy++;
        if(cur_win->blk_cy==cur_win->blk_count_y)
            cur_win->blk_cx=cur_win->blk_cy=0;
    }
    gfx_mv_cur(cur_win->blk_cx,cur_win->blk_cy);
}

int gfx_printf(const char* format, ...)
{
    va_list ap;
    int ret=0;
    int n=0;
    int i;
    char buf[MAX_PRINTLEN];
    SDL_Surface* icon;
    if(!cur_win){
        fprintf(stderr,"gfx_printf: cur_win == NULL!\n");
        return 0;
    }
    va_start(ap, format);
    ret=vsnprintf(buf,MAX_PRINTLEN-1,format,ap);
    va_end(ap);
    while(buf[n]&&n<MAX_PRINTLEN){
        if(buf[n]=='\n'){
            n++;
            gfx_inc_cursor(TRUE);
            continue;
        }
        i=char_to_icon(buf[n]);
        icon=(cur_win->half_size?hs_icons[i]:icons[i]);
        draw_image(icon,cur_win->win_cx,cur_win->win_cy);
        gfx_inc_cursor(FALSE);
        n++;
    }
    return n;
}

void delay(unsigned int flimit)
{
    unsigned int ticks=SDL_GetTicks();
    if (flimit<ticks)
        return;
    if (flimit>ticks+TICK_COUNT)
        SDL_Delay(TICK_COUNT);
    else
        SDL_Delay(flimit-ticks);
}

void gfx_message(const char* format, ...)
{
    va_list ap;
    int x,y;
    int n=0;
    bool half_size=cur_win->half_size;
    char buf[MAX_PRINTLEN];
    if(!cur_win){
        fprintf(stderr,"gfx_message: cur_win == NULL!\n");
        return;
    }
    va_start(ap, format);
    n=vsnprintf(buf,MAX_PRINTLEN-1,format,ap);
    va_end(ap);
    if(!half_size)
        gfx_win_half_size();
    x=(cur_win->blk_count_x-strlen(buf))/2;
    y=cur_win->blk_count_y/2+2;
    gfx_box(x-1,y-1,x+n,y+1,TRUE);
    set_space_icon(SPC_OPAQUE);
    gfx_mv_cur(x,y);
    gfx_printf(buf);
    if(!half_size)
        gfx_win_full_size();
    SDL_Flip(screen);
    wait_till_pressed();
}

void gfx_box(int tlx, int tly, int brx, int bry, bool solid)
{
    int x,y;
    for(x=tlx;x<=brx;x++){
        gfx_win_draw_image(hs_icons[ICON_LINE_T],x, tly);
        gfx_win_draw_image(hs_icons[ICON_LINE_B],x, bry);
    }
    for(y=tly;y<=bry;y++){
        gfx_win_draw_image(hs_icons[ICON_LINE_L],tlx,y);
        gfx_win_draw_image(hs_icons[ICON_LINE_R],brx,y);
    }
    gfx_win_draw_image(hs_icons[ICON_LINE_TL],tlx,tly);
    gfx_win_draw_image(hs_icons[ICON_LINE_TR],brx,tly);
    gfx_win_draw_image(hs_icons[ICON_LINE_BL],tlx,bry);
    gfx_win_draw_image(hs_icons[ICON_LINE_BR],brx,bry);
    if(solid){
        for(y=tly+1;y<bry;y++)
            for(x=tlx+1;x<brx;x++)
                gfx_win_draw_image(hs_icons[ICON_SPACE_OPAQUE],x,y);
    }
}
