/*
    XorGramana Copyright 2009 James W. Morris, james@jwm-art.net

    This file is part of XorGramana.

    XorGramana is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    XorGramana is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with XorGramana.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _GFX_H
#define _GFX_H

#include <SDL.h>
#include "defs.h"
#include "types.h"
#include "icons.h"

/* Please don't make this any smaller */
#define GFX_MIN_WIDTH   800
#define GFX_MIN_HEIGHT  600

struct gfx_mode
{
    SDL_Rect **modes;
    int mode;
    SDL_Rect* m;
    int def;
    int last;
};

/*  this window implimentation is very simple, and no clipping      *
 *  takes place. all windows write to the main SDL_Surface (screen) *
 *  and they are only used for positioning text (albeit text        *
 *  represented by the very same icons). most of the code has been  *
 *  written for the textual layout of the level menu etc, and       *
 *  to provide various information during game play. the game code  *
 *  won't use the text abstraction (but will make use of window     *
 *  coordinates for positioning, etc.)                              */

enum NEW_WIN_FLAGS
{
    GFX_DEF=        0x00,   /*  default, use exact coords passed    */
    GFX_CENTER_X=   0x01,   /*  horizontally center window          */
    GFX_CENTER_Y=   0x02,   /*  vertically center window            */
    GFX_ALIGN_R=    0x04,   /*  align window right                  */
    GFX_ALIGN_B=    0x08,   /*  align window bottom                 */
    GFX_BLK_ALIGN_X=0x10,   /*  any of above, snap to horiz blocks  */
    GFX_BLK_ALIGN_Y=0x20    /*  any of above, snap to vert blocks   */
};

/*  note: the block align enumerations will mess up centering when  *
 *  the size of the window is only marginally smaller than the      *
 *  screen.                                                         */

struct gfx_win
{
    /*  window coords (pixels)              */
    int win_tlx;
    int win_tly;
    int win_brx;
    int win_bry;
    int win_w;
    int win_h;
    /*  window cursor pos                   */
    int win_cx;
    int win_cy;
    /*  block/icon dimensions               */
    int icon_width;
    int icon_height;
    /*  block/icon display system pixel     *
     *  coords within window                */
    int blk_tlx;
    int blk_tly;
    int blk_brx;
    int blk_bry;
    /*  block cursor pos                    */
    su_t blk_cx;
    su_t blk_cy;
    su_t blk_count_x;
    su_t blk_count_y;
    bool half_size;
    SDL_Surface** icons;
};

extern SDL_Surface *screen;
extern struct gfx_mode kermode;
extern struct gfx_win* scr_win;  /* default window (border)*/
extern struct gfx_win* main_win; /* main window (within border)*/
extern struct gfx_win* cur_win;

void gfx_start();
void gfx_end();

void scr_win_border();

void gfx_inc_mode();
void gfx_dec_mode();

/*--------------------------------------------------*
 *  these functions ignore window structures        */

void overlay_image(SDL_Surface* image, SDL_Surface *overlay, int x, int y);

void draw_image(SDL_Surface *image, int x, int y);

/*  lifted from SDL manual, used by icon scaling    */
Uint32 getpixel(SDL_Surface *surface, int x, int y);
void putpixel(SDL_Surface *surface, int x, int y, Uint32 pixel);

/*--------------------------------------------------*
 *  window related functions follow:                */

/*  note: dimensions of image used as block size, but in    *
 *  xorgramana these are same dimensions (or half ;-)       */
void gfx_win_draw_image(SDL_Surface *image, int blk_x, int blk_y);

struct gfx_win*
gfx_new_win(int flags,
            int tlx, int tly,int brx, int bry,
            int blk_width,int blk_height);

/*  set window to operate on                        */
void gfx_set_win(struct gfx_win* win);

/*  change icon sizes to display for current window */
void gfx_win_half_size();
void gfx_win_full_size();

/*  move the cursor to location block x,y           */
su_t gfx_mv_cur(su_t blk_x, su_t blk_y);

/*  guess   */
int gfx_printf(const char* format, ...);

void delay(unsigned int flimit);

void gfx_message(const char* format, ...);
void gfx_box(int tlx,int tly,int brx,int bry,bool solid);
#endif
