/*
    XorGramana Copyright 2009 James W. Morris, james@jwm-art.net

    This file is part of XorGramana.

    XorGramana is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    XorGramana is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with XorGramana.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _ICONS_H
#define _ICONS_H

#include "types.h"
#include "defs.h"

#include <SDL.h>
#include <SDL_image.h>

/*  noof wall icons - the wall icon points to one of these,         *
 *  depending on which level is played                              */
#define WALL_XXX 16
#define EDGE_XXX 4

/*  the icon structure has changed quite substantially from that    *
 *  in  xorcurses. firstly, using SDL, there is only a pointer to   *
 *  the image data, whereas before there was a character array.     *
 *  now, the SDL_Surface pointer has been removed from the struct   *
 *  and an array created instead. this is so a second array can be  *
 *  created which will contain half-size icons.  this will allow    *
 *  easy switching between the two sets of icons by display code.   */

struct xg_icon
{
    char* fname;
    char mapchar;
};

enum ICON
{
    ICON_FLOOR,
    ICON_WALL,
    ICON_H_FIELD,
    ICON_V_FIELD,
    ICON_PLAYER0,
    ICON_PLAYER1,
    ICON_MAP,
    ICON_TELEPORT,
    ICON_EXIT,
    ICON_MASK,
    ICON_DOT,
    ICON_BLANK,
    /* now follows icons with indicatable behaviours */
    ICON_A,
    ICON_B,
    ICON_C,
    ICON_D,
    ICON_E,
    ICON_F,
    ICON_G,
    ICON_H,
    ICON_I,
    ICON_J,
    ICON_K,
    ICON_L,
    ICON_M,
    ICON_N,
    ICON_O,
    ICON_P,
    ICON_Q,
    ICON_R,
    ICON_S,
    ICON_T,
    ICON_U,
    ICON_V,
    ICON_W,
    ICON_X,
    ICON_Y,
    ICON_Z,
    ICON_0,
    ICON_1,
    ICON_2,
    ICON_3,
    ICON_4,
    ICON_5,
    ICON_6,
    ICON_7,
    ICON_8,
    ICON_9,
/* end of icons with indicatable behaviours */
/***** THE FOLLOWING ARE NOT SPECIFIABLE IN MAP *****/
    ICON_EXIT_OPEN,
    ICON_SPACE_TRAN,
    ICON_SPACE_OPAQUE,
    ICON_EXPLOSION1,
    ICON_EXPLOSION2,
    ICON_EXPLOSION3,
    ICON_EXPLOSION4,
    ICON_EXPLOSION5,
    ICON_LINE_L,
    ICON_LINE_R,
    ICON_LINE_T,
    ICON_LINE_B,
    ICON_LINE_TL,
    ICON_LINE_TR,
    ICON_LINE_BL,
    ICON_LINE_BR,
    ICON_LOADED_XXX, /* end of icons loaded from files */
/* generated */
    ICON_EDGE_T,
    ICON_EDGE_B,
    ICON_EDGE_L,
    ICON_EDGE_R,
    ICON_EDGE_TB,
    ICON_EDGE_LR,
    ICON_EDGE_TL,
    ICON_EDGE_TR,
    ICON_EDGE_BL,
    ICON_EDGE_BR,
    ICON_EDGE_TBL,
    ICON_EDGE_TBR,
    ICON_EDGE_TLR,
    ICON_EDGE_BLR,
    ICON_EDGE_TBLR,
    ICON_XXX
};

enum DIRIND
{
    DIRIND_GRAV_U,
    DIRIND_GRAV_D,
    DIRIND_GRAV_L,
    DIRIND_GRAV_R,
    DIRIND_PUSH_U,
    DIRIND_PUSH_D,
    DIRIND_PUSH_L,
    DIRIND_PUSH_R,
    DIRIND_HARDPUSH_U,
    DIRIND_HARDPUSH_D,
    DIRIND_HARDPUSH_L,
    DIRIND_HARDPUSH_R,
    DIRIND_EXPLODE_H,
    DIRIND_EXPLODE_V,
    DIRIND_XXX
};

enum EDGES
{
    EDGE_T=     0x01,
    EDGE_B=     0x02,
    EDGE_L=     0x04,
    EDGE_R=     0x08,
    EDGE_TB=    EDGE_T|EDGE_B,
    EDGE_LR=    EDGE_L|EDGE_R,
    EDGE_TL=    EDGE_T|EDGE_L,
    EDGE_TR=    EDGE_T|EDGE_R,
    EDGE_BL=    EDGE_B|EDGE_L,
    EDGE_BR=    EDGE_B|EDGE_R,
    EDGE_TBL=   EDGE_TB|EDGE_L,
    EDGE_TBR=   EDGE_TB|EDGE_R,
    EDGE_TLR=   EDGE_TL|EDGE_R,
    EDGE_BLR=   EDGE_BL|EDGE_R,
    EDGE_TBLR=  EDGE_TB|EDGE_LR
};

enum SPACE_CHAR
{
    SPC_OPAQUE,
    SPC_TRAN,
    SPC_FLOOR
};

/* names of icons, the ICON_EXT is added to the name to         *
 * to determine the file name. icon_details holds the name and  *
 * corresponding map character (if any) for the main icon bank. */

extern struct xg_icon icon_details[ICON_XXX];
extern char* wall_names[WALL_XXX];
extern char* floor_names[WALL_XXX];
extern char* edge_names[WALL_XXX][EDGE_XXX];
extern char* indicator_names[DIRIND_XXX];

/* these store all the different wall and floor patterns        *
 * current_wall_textures stores the index of the current wall   */
extern SDL_Surface* walls[WALL_XXX];
extern SDL_Surface* floors[WALL_XXX];
extern SDL_Surface* hs_walls[WALL_XXX];
extern SDL_Surface* hs_floors[WALL_XXX];
extern ss_t current_wall_texture;

/* wall_edges stores all the different edge images for all the  *
 * different wall images - these get overlaid to create the     *
 * edged_wall patterns.                                         */
extern SDL_Surface* wall_edges[WALL_XXX][EDGE_XXX];

/* indicators stores the indicator images which are overlaid    *
 * upon those icons which require behaviour indicators.         */
extern SDL_Surface* indicators[DIRIND_XXX];

/* edged_walls stores the current wall pattern with each        *
 * possible combination of the four edges for that pattern      *
 * overlaid onto it.                                            */
extern SDL_Surface* edged_walls[ICON_EDGE_TBLR-ICON_EDGE_T];


/* icons holds the main icon bank                               *
 * the main icon bank points to the icons currently to be used. *
 * wall and floor icons depend on which level is being played   *
 * the pointers in the main icon bank point to what is needed   *
 * the switch object turns the wall visibility off, which is    *
 * acheived by setting all the wall pointers to point to the    *
 * floor icon.                                                  */

extern SDL_Surface* icons[ICON_XXX];
extern SDL_Surface* hs_icons[ICON_XXX];

/* icons with indicators duplicates a limited set of icons from *
 * the main icon bank and overlays the behaviour indicators     *
 * onto them. icons not requiring indicators are not duplicated.*/
extern SDL_Surface* icons_with_indicators[ICON_XXX];


extern ss_t current_wall_texture;
extern enum ICON spc_icon;

void init_icons();
void init_wall(lvl_t level, bool show);

void icons_cleanup();

su_t mapchar_to_icon(char c);

/* char to icon called by gfx_printf - ie converts ASCII char   *
 * to ICON enumeration. The result of sending a ' ' (space) to  *
 * this function can be controlled by set_space_char.           */
su_t char_to_icon(char c);
void set_space_icon(enum SPACE_CHAR);

/* loading icon */
SDL_Surface *load_image(char *name, bool transparent, int tr_r, int tr_g, int tr_b);
SDL_Surface* icon_duplicate(SDL_Surface* s, bool transparent, int tr_r, int tr_g, int tr_b);
SDL_Surface* icon_half_assed_size(SDL_Surface* s, bool transparent, int tr_r, int tr_g, int tr_b);

#endif /* _ICONS_H */

