/*
    XorGramana Copyright 2009 James W. Morris, james@jwm-art.net

    This file is part of XorGramana.

    XorGramana is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    XorGramana is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with XorGramana.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "input.h"
#include "gfx.h"

struct xg_input xginput={
    0,
    SDL_DEFAULT_REPEAT_DELAY/3,
    SDL_DEFAULT_REPEAT_INTERVAL*2,
    0,0
};

struct xg_key** create_key_list_array(int count)
{
    int n;
    struct xg_key** keylist;
    if(!(keylist=calloc(count+1,sizeof(struct xg_key)))){
        fprintf(stderr,"Failed to allocate list for %d key data items\n",count);
        return 0;
    }
    for(n=0;n<count;n++){
        if(!(keylist[n]=calloc(1,sizeof(struct xg_key)))){
            fprintf(stderr,
                "Failed to allocate list for %d key data items\n",
                count);
            free(keylist);
            return 0;
        }
    }
    keylist[count]=0;
    return keylist;
}

void destroy_key_list_array(struct xg_key** keylist)
{
    struct xg_key* key=0;
    int n;
    if(!keylist)
        return;
    n=0;
    #ifdef INPUT_DEBUG
    printf("Freeing keylistarray:%lx",(unsigned long)keylist);
    #endif
    while((key=keylist[n])!=0){
        #ifdef INPUT_DEBUG
        printf("freeing key: %c\n",key->key);
        #endif
        free(key);
        n++;
    }
    #ifdef INPUT_DEBUG
    printf("that list done.\n");
    #endif
    free(keylist);
}

void set_keys(struct xg_key** keys)
{
    xginput.keys=keys;
}

void key_repeat_on()
{
    SDL_EnableKeyRepeat(xginput.r_delay,xginput.r_interval);
}

void key_repeat_off()
{
    SDL_EnableKeyRepeat(0,0);
}

bool simple_poll_event()
{
    SDL_Event event;
    SDLKey sym;
    struct xg_key* key=0;
    int n;
    if(!xginput.keys)
        return FALSE;
    xginput.quit=0;
    xginput.exit=0;
    for(n=0;(key=xginput.keys[n])!=0;n++)
        key->pressed=0;
    while(SDL_PollEvent(&event)){
        switch(event.type){
            case SDL_QUIT:
                xginput.exit=1;
                return TRUE;
            case SDL_KEYDOWN:
                sym=event.key.keysym.sym;
                if(sym==SDLK_ESCAPE){
                    xginput.quit=1;
                    return TRUE;
                }
                else if(sym==SDLK_F11){
                    SDL_WM_ToggleFullScreen(screen);
                    return TRUE;
                }
                else{
                    for(n=0;(key=xginput.keys[n])!=0;n++){
                        if(sym==key->key){
                            key->pressed=1;
                            return TRUE;
                        }
                    }
                }
                break;
            default:
                break;
        }
    }
    return FALSE;
}

void wait_till_pressed()
{
    unsigned int flimit=SDL_GetTicks()+TICK_COUNT;
    while(!simple_poll_event()){
        delay(flimit);
        flimit=SDL_GetTicks()+TICK_COUNT;
    }
}
