/*
    XorGramana Copyright 2009 James W. Morris, james@jwm-art.net

    This file is part of XorGramana.

    XorGramana is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    XorGramana is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with XorGramana.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "level_menu.h"

#include "map.h"
#include "scores.h"

#include "input.h"
#include "version.h"

#include <string.h>
#include <stdlib.h>

struct gfx_win* level_win=0;
struct xg_key** menu_keys=0;

void menu_splash();

void level_win_create()
{
    /* utilizing int's loss of fractions */
    int rx=main_win->win_w;
    int by=main_win->win_h;
    int flags=GFX_CENTER_X|GFX_CENTER_Y;
    if(main_win->win_w>GFX_MIN_WIDTH){
        rx= ((main_win->win_w/ICON_WIDTH)
            - (2*(main_win->win_w/GFX_MIN_WIDTH))
            ) * ICON_WIDTH;
        flags|=GFX_BLK_ALIGN_X;
    }
    if(main_win->win_h>GFX_MIN_HEIGHT){
        by= ((main_win->win_h/ICON_HEIGHT)
            - (2*(main_win->win_h/GFX_MIN_HEIGHT))
            ) * ICON_HEIGHT;
        flags|=GFX_BLK_ALIGN_Y;
    }
    level_win=gfx_new_win(flags,0,0,rx,by,ICON_WIDTH,ICON_HEIGHT);
    if(!(menu_keys=create_key_list_array(MENU_KEYS_XXX)))
        return;
    menu_keys[MENU_UP]->key=            SDLK_UP;
    menu_keys[MENU_DOWN]->key=          SDLK_DOWN;
    menu_keys[MENU_PLAY]->key=          SDLK_RETURN;
    menu_keys[MENU_LOAD_REPLAY]->key=   SDLK_l;
    menu_keys[MENU_OPTIONS]->key=       SDLK_o;
    menu_keys[MENU_SWITCH_GAME]->key=   SDLK_x;
}

void level_win_destroy()
{
    free(level_win);
    #ifdef MISC_DEBUG
    printf("about to destroy menu_keys\n");
    #endif
    destroy_key_list_array(menu_keys);
    menu_keys=0;
}

lvl_t level_menu(lvl_t level)
{
    int i,lvl;
    int first_level=-1;
    int last_level=0;
    int level_count=0;
    int show=8;
    int show_from;
    unsigned int flimit=SDL_GetTicks()+TICK_COUNT;
    set_keys(menu_keys);
    for(i=MIN_LEVEL;i<=MAX_LEVEL;i++){
        if(map_name[i]){
            if(first_level==-1)
                first_level=i;
            else
                last_level=i;
        }
    }
    #ifdef MISC_DEBUG
    printf("level=%d\n",level);
    printf("first_level:%d last_level:%d\n",first_level,last_level);
    #endif
    level_count=last_level-first_level;
    if(show>level_count)
        show=level_count+1;
    show_from=level-show/2;
    #ifdef MISC_DEBUG
        printf("show_from:%d(initialised to)\n",show_from);
    #endif
    if(show_from<first_level)
        show_from=first_level;
    else if(show_from+show>=last_level)
        show_from=last_level-(show-1);
    #ifdef MISC_DEBUG
        printf("show_from:%d(set to)\n",show_from);
    #endif
    menu_splash();
    for(i=0;i<show;i++){
        gfx_mv_cur(0,4+i);
        lvl=show_from+i;
        if(level==lvl)
            gfx_printf("-%s-",map_name[lvl]);
        else
            gfx_printf(" %s",map_name[lvl]);
    }
    SDL_Flip(screen);
    do{
        if(simple_poll_event()){
            if(xginput.quit)
                xginput.exit=1;
            if(xginput.exit)
                return -1;
            if(menu_keys[MENU_UP]->pressed){
                if(level>first_level){
                    level--;
                    if((level==show_from)&&(show_from>first_level))
                        show_from--;
                }
            }
            else if(menu_keys[MENU_DOWN]->pressed){
                if(level<last_level){
                    level++;
                    if((level==show_from+show)&&(show_from+show<=last_level))
                        show_from++;
                }
            }
            else if(menu_keys[MENU_PLAY]->pressed)
                return level;
            else if(menu_keys[MENU_SWITCH_GAME]->pressed){
                set_game(options->game^1);
                for(i=MIN_LEVEL;i<=MAX_LEVEL;i++){
                    if(map_name[i]){
                        if(first_level==-1)
                            first_level=i;
                        else
                            last_level=i;
                    }
                }
                level=show_from=first_level;
                level_count=last_level-first_level;
                if(level_count>7)
                    show=8;
                else
                    show=level_count+1;
                #ifdef MISC_DEBUG
                printf("first_level:%d last_level:%d\n",first_level,last_level);
                #endif
            }
            #ifdef MISC_DEBUG
            printf("level:%d\n",level);
            #endif
            menu_splash();
            for(i=0;i<show;i++){
                gfx_mv_cur(0,4+i);
                lvl=show_from+i;
                if(level==lvl)
                    gfx_printf("-%s-",map_name[lvl]);                
                else
                    gfx_printf(" %s",map_name[lvl]);
            }
            SDL_Flip(screen);
        }
        delay(flimit);
        flimit=SDL_GetTicks()+TICK_COUNT;
    }while(1);
}

void menu_splash()
{
    char spc[80];
    int gx=(options->game?3:12);
    int x;
    int n;
    init_wall(1,TRUE);
    for(n=0;n<80;spc[n++]=' ');
    spc[main_win->blk_count_x]='\0';
    gfx_set_win(main_win);
    gfx_mv_cur(0,0);
    gfx_win_full_size();
    set_space_icon(SPC_FLOOR);
    for(n=0;n<main_win->blk_count_y;n++)
        gfx_printf("%s",spc);
    gfx_set_win(level_win);
    gfx_win_full_size();
    x=(level_win->blk_count_x-gx)/2;
    gfx_mv_cur(x,0);
    gfx_printf((options->game?"Xor":"XorGramana"));
    if(options->game)
        x=0;
    gfx_mv_cur(0,1);
    gfx_win_half_size();
    spc[main_win->blk_count_x]='_';
    spc[level_win->blk_count_x]='\0';
    for(n=0;n<level_win->blk_count_y-2;n++)
        gfx_printf("%s",spc);
    if(options->game){
        gfx_mv_cur(x+1,2);
        gfx_printf("remake by jwm-art.net");
    }
    else{
        gfx_mv_cur(x+2,2);
        gfx_printf("%s by jwm-art.net",verstr);
    }
    gfx_mv_cur(x,level_win->blk_count_y-3);
    gfx_printf("x-switch game");
    gfx_mv_cur(x,level_win->blk_count_y-2);
    gfx_printf("l-load replay");
    gfx_mv_cur(x,level_win->blk_count_y-1);
    gfx_printf("o-options");
}

void help()
{
    /*
    char* hf;
    FILE* fp;
    char help[120][43];
    xy_t r,maxrows;
    xy_t row=0;
    int key;
    if(!(hf=options_file_path("xorcurseshelp.txt",options->data_dir)))
        return;
    if((fp=fopen(hf, "r"))){
        while(fgets(help[row++], 42, fp)&&row<120);
        free(hf);
        fclose(fp);
        maxrows=row-1;
    }
    else{
        free(hf);
        icon_help();
        return;
    }
    row=0;
    wclear(game_win);
    wattrset(game_win,COLOR_PAIR(0));
    do{
        for(r=0;r<24;r++)
            mvwprintw(game_win,r,0,help[r+row]);
        switch((key=wgetch(game_win)))
        {
            case KEY_UP:    if(row>0)row--;break;
            case KEY_DOWN:  if(row+24<maxrows)row++;break;
            case 'q':
            case 'Q':
                return;
        }
    }while(key!='h'&&key!='H');
    icon_help();
    */
}

void icon_help()
{
    /*
    char* hf;
    FILE* fp;
    char help[50][43];
    xy_t row=0;
    xy_t irow=0;
    xy_t r,maxrows;
    int key;
    su_t i;
    if(!(hf=options_file_path("xorcursesiconhelp.txt",options->data_dir)))
        return;
    if((fp=fopen(hf, "r"))){
        while(fgets(help[row++], 42, fp)&&row<50);
        free(hf);
        fclose(fp);
        maxrows=row-1;
    }
    else{
        free(hf);
        return;
    }
    wclear(game_win);
    row=0;
    do{
        wattrset(game_win,COLOR_PAIR(0));
        for(r=0;r<24;r++)
            mvwprintw(game_win,r,0,help[r+row]);
        for(i=0;i<12;i++){
            game_win_icon_dump(0,i*4-row,i+ICON_H_FIELD);
        }
        irow=(row+1)/4;
        switch((key=wgetch(game_win)))
        {
            case KEY_UP:    if(row>0)row--;break;
            case KEY_DOWN:  if(row+24<maxrows)row++;break;
            case 'q': case 'Q': case 'h': case 'H':
                return;
        }
    }while(key!='q'&&key!='Q');
    */
}
