/*
    XorGramana Copyright 2009 James W. Morris, james@jwm-art.net

    This file is part of XorGramana.

    XorGramana is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    XorGramana is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with XorGramana.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _MAP_H
#define _MAP_H

#include "types.h"
#include "defs.h"
#include <stdio.h>

struct xy
{
    xy_t x;
    xy_t y;
};

struct xg_map
{
    char*       filename;
    char*       title;
    ctr_t       max_moves;  /* number of moves allowed to complete a level */
    su_t        width;
    su_t        height;
    map_t**     data;
    struct xy   view[2];    /* start coords of initial player views (gen)  */
    struct xy   mappc[4];   /* coords of map piece 0 1 2 3                  */
    struct xy   player[2];      /* positions - by scanning map-data */
    struct xy   teleport[2];    /* positions, scanned               */
    struct xy   exit;           /* exit position */
    su_t        level;          /* user's choice */
    su_t        mask_count;
    map_t*      solve_word[MAX_WORDS_TO_SOLVE+1];
    struct xy   solve_xy[MAX_WORDS_TO_SOLVE+1];
    xy_t        solve_y[MAX_WORDS_TO_SOLVE+1];
    su_t        solve_dir[MAX_WORDS_TO_SOLVE+1];
    struct xy   solve_unblock_xy[MAX_WORDS_TO_SOLVE+1];
    bool        solved[MAX_WORDS_TO_SOLVE+1];
};

/*  the solutions take the form of:                             *
 *  1) a list of words                                          *
 *  2) a list of x,y positions for first letter of each word    *
 *  3) a list of directions right,down how each word should be  *
 *     placed.                                                  *
 *  (once the player has arranged letters such that these words *
 *   can be read at the respective positions in the map, then   *
 *   player can finish the level and go through the exit.)      */

extern struct xg_map* map;

void xg_map_create(const char* title, int width,int height);
void xg_map_destroy();
void xg_map_load(su_t level);

void xg_map_init_views(xy_t view_w, xy_t view_h);

char* xg_map_read_name(FILE* fp);
char* xg_map_load_read_name(su_t level);

/*  map_get_teleport checks the coordinates passed against
    those in the map->teleports array and returns 0 if
    coordinate is teleport 0, and 1 if coordinate is teleport
    1. returns 99 if coord is not a teleport.
*/
su_t map_get_teleport(xy_t x, xy_t y);

/* brute force exit(1) */
void xg_map_load_error(FILE* fp, char* filename, char* msg);

void xg_map_apply_edging();

bool is_wall(xy_t x, xy_t y);

/*  test map against word solutions, return TRUE if all matches */
bool map_check_solved();

#ifdef MAPDEVMODE
void map_dump_solve_state();
#endif

#endif /* _MAP_H */
