#include "mapdisplay.h"
#include "map.h"
#include "game_display.h"
#include "play_xor.h"
#include "options.h"


struct map_display_data map_display={{0,0},0,0,0,0,0,0,0,0,0,{0},{0}};
struct xg_key** map_keys=0;

void map_display_calc_coords();
void map_display_draw();

su_t map_display_icon_to_icon(su_t i)
{
    switch(i){
        case MAP_FLOOR:         return  ICON_FLOOR;
        case MAP_WALL:          return  ICON_WALL;
        case MAP_MASK:          return  ICON_MASK;
        case MAP_EXIT:          return  ICON_EXIT;
        case MAP_EXIT_OPEN:     return  ICON_EXIT_OPEN;
        case MAP_BLANK:         return  ICON_BLANK;
        case MAP_SPACE:         return  ICON_SPACE_OPAQUE;
        default:                return  ICON_FLOOR;
    }
}

su_t icon_to_map_display_icon(su_t i)
{
    switch(i){
        case ICON_FLOOR:        return  MAP_FLOOR;
        case ICON_WALL:         return  MAP_WALL;
        case ICON_MASK:         return  MAP_MASK;
        case ICON_EXIT:         return  MAP_EXIT;
        case ICON_EXIT_OPEN:    return  MAP_EXIT_OPEN;
        case ICON_BLANK:        return  MAP_BLANK;
        case ICON_SPACE_OPAQUE: return  MAP_SPACE;
        default:                return  MAP_FLOOR;
    }
}

void map_display_init()
{
    if(!map||!game_win)
        return;
    map_display.center.x=map->width/2;
    map_display.center.y=map->height/2;
    map_display.zoom=16;
    map_display.zicons=map_display.icons16;
    map_display.min_zoom=8;
    int i;
    bool trans;
    map_display_destroy_icons();
    for(i=0;i<MAP_ICONS_XXX;i++){
        trans=(i<=MAP_WALL||i==MAP_SPACE?FALSE:TRUE);
        map_display.icons32[i]=hs_icons[map_display_icon_to_icon(i)];
        #ifdef MISC_DEBUG
        printf("map_display.icons32[%d]:%lx\n",i,(unsigned long)map_display.icons32[i]);
        #endif
        map_display.icons16[i]=
            icon_half_assed_size(map_display.icons32[i],trans,0,0,0);
        if(!map_display.icons16[i]){
            map_display.zoom=map_display.min_zoom=32;
            return;
        }
    }
    for(i=0;i<MAP_ICONS_XXX;i++){
        trans=(i<=MAP_WALL||i==MAP_SPACE?FALSE:TRUE);
        map_display.icons8[i]=
            icon_half_assed_size(map_display.icons16[i],trans,0,0,0);
        if(!map_display.icons8[i]){
            map_display.min_zoom=16;
            return;
        }
    }
}

void map_display_destroy_icons()
{
    int i;
    for(i=0;i<MAP_ICONS_XXX;i++){
        if(map_display.icons16[i])
            SDL_FreeSurface(map_display.icons16[i]);
        if(map_display.icons8[i])
            SDL_FreeSurface(map_display.icons8[i]);
    }
}

bool map_display_create_keys()
{
    if(!(map_keys=create_key_list_array(MAP_KEYS_XXX)))
        return FALSE;
    map_keys[MAP_KEY_UP]->key=      SDLK_UP;
    map_keys[MAP_KEY_DOWN]->key=    SDLK_DOWN;
    map_keys[MAP_KEY_LEFT]->key=    SDLK_LEFT;
    map_keys[MAP_KEY_RIGHT]->key=   SDLK_RIGHT;
    map_keys[MAP_KEY_IN]->key=      SDLK_EQUALS;
    map_keys[MAP_KEY_OUT]->key=     SDLK_MINUS;
    map_keys[MAP_KEY_HIDE]->key=    SDLK_m;
    return TRUE;
}

void map_display_destroy_keys()
{
    destroy_key_list_array(map_keys);
}

void map_display_show()
{
    unsigned int flimit=SDL_GetTicks()+TICK_COUNT;
    if(!player.have_map)
        return;
    set_keys(map_keys);
    map_display_calc_coords();
    map_display_draw();
    do{
        if(simple_poll_event()){
            if(xginput.quit||xginput.exit||map_keys[MAP_KEY_HIDE]->pressed){
                set_keys(play_keys);
                game_win_display();
                return;
            }
            if(map_keys[MAP_KEY_UP]->pressed){
                if(map_display.map_tly>0)
                    map_display.center.y--;
            }
            else if(map_keys[MAP_KEY_DOWN]->pressed){
                if(map_display.map_bry<map->height)
                    map_display.center.y++;
            }
            else if(map_keys[MAP_KEY_LEFT]->pressed){
                if(map_display.map_tlx>0)
                    map_display.center.x--;
            }
            else if(map_keys[MAP_KEY_RIGHT]->pressed){
                if(map_display.map_brx<map->width)
                    map_display.center.x++;
            }
            else if(map_keys[MAP_KEY_OUT]->pressed){
                if(map_display.zoom>map_display.min_zoom)
                    map_display.zoom/=2;
            }
            else if(map_keys[MAP_KEY_IN]->pressed){
                if(map_display.zoom<32)
                    map_display.zoom*=2;
            }
            map_display_calc_coords();
            map_display_draw();
        }
        delay(flimit);
        flimit=SDL_GetTicks()+TICK_COUNT;
    }while(1);
}

void map_display_calc_coords()
{
    int cx=map_display.center.x;
    int cy=map_display.center.y;
    switch(map_display.zoom){
        case 32:    map_display.zicons=map_display.icons32; break;
        case 16:    map_display.zicons=map_display.icons16; break;
        case 8:     map_display.zicons=map_display.icons8;  break;
    }
    map_display.blk_count_x=(game_win->blk_count_x*game_win->icon_width)/map_display.zoom;
    map_display.blk_count_y=(game_win->blk_count_y*game_win->icon_height)/map_display.zoom;
    if(map_display.blk_count_x>=map->width)
        cx=map->width/2;
    if(map_display.blk_count_y>=map->height)
        cy=map->height/2;
    map_display.map_tlx=cx-map_display.blk_count_x/2;
    map_display.map_tly=cy-map_display.blk_count_y/2;
    map_display.map_brx=map_display.map_tlx+map_display.blk_count_x;
    map_display.map_bry=map_display.map_tly+map_display.blk_count_y;
    #ifdef MISC_DEBUG
    printf("map_display:\nzoom:%d\ncenter.x:%d center.y:%d\n",
        map_display.zoom,map_display.center.x,map_display.center.y);
    printf("blk_count_x:%d blk_count_y:%d\n",
        map_display.blk_count_x,map_display.blk_count_y);
    printf("map_tlx:%d map_tly:%d\nmap_brx:%d map_bry:%d\n",
        map_display.map_tlx,map_display.map_tly,
        map_display.map_brx,map_display.map_bry);
    #endif
}

/* map display sections

  8 | 1
  --+--
  4 | 2

*/

void map_display_draw()
{
    xy_t x,y;
    int mx,my;
    su_t icon;
    xy_t h_mx=map->width/2;
    xy_t h_my=map->height/2;
    for(y=0;y<map_display.blk_count_y;y++){
        my=y+map_display.map_tly;
        for(x=0;x<map_display.blk_count_x;x++){
            mx=x+map_display.map_tlx;
            if(mx<h_mx){
                if(my<h_my){
                    if(!(player.have_map&8))
                        mx=-1;
                }
                else if(!(player.have_map&4))
                    mx=-1;
            }
            else if(my<h_my){
                if(!(player.have_map&1))
                    mx=-1;
            }
            else if(!(player.have_map&2))
                mx=-1;
            if(my>=0&&my<map->height&&mx>=0&&mx<map->width){
                draw_image(map_display.zicons[MAP_FLOOR],
                    game_win->blk_tlx+x*map_display.zoom,
                    game_win->blk_tly+y*map_display.zoom);
                icon=map->data[my][mx];
                if(icon==ICON_MASK&&!options->game)
                    icon=ICON_FLOOR;
                if(icon!=ICON_FLOOR){
                    if(icon>=ICON_EDGE_T&&icon<=ICON_EDGE_TBLR)
                        icon=ICON_WALL;
                    draw_image(
                        map_display.zicons[icon_to_map_display_icon(icon)],
                        game_win->blk_tlx+x*map_display.zoom,
                        game_win->blk_tly+y*map_display.zoom);
                }
            }
            else{
                draw_image(map_display.zicons[MAP_SPACE],
                    game_win->blk_tlx+x*map_display.zoom,
                    game_win->blk_tly+y*map_display.zoom);
            }
        }
    }
    if(options->game){
        SDL_Flip(screen);
        return;
    }
    su_t n=0;
    map_t* p;
    while((p=map->solve_word[n])){
        if(!map->solved[n]){
            x=map->solve_xy[n].x;
            y=map->solve_xy[n].y;
            while(*p){
                mx=x-map_display.map_tlx;
                my=y-map_display.map_tly;
                if(x<h_mx){
                    if(y<h_my){
                        if(!(player.have_map&8))
                            mx=-1;
                    }
                    else if(!(player.have_map&4))
                        mx=-1;
                }
                else if(y<h_my){
                    if(!(player.have_map&1))
                        mx=-1;
                }
                else if(!(player.have_map&2))
                    mx=-1;
                if(my>=0&&my<map_display.blk_count_y
                 &&mx>=0&&mx<map_display.blk_count_x)
                {
                    draw_image(
                        map_display.zicons[MAP_BLANK],
                            game_win->blk_tlx+mx*map_display.zoom,
                            game_win->blk_tly+my*map_display.zoom);
                }
                if(map->solve_dir[n]==MV_DOWN)
                    y++;
                else
                    x++;
                p++;
            }
        }
        n++;
    }
    SDL_Flip(screen);
}

void game_win_dump_map_sect(xy_t topy, su_t sect, bool show)
{
/*
    xy_t mapstx,mapsty,mapmaxx,mapmaxy;
    xy_t x,y,icon;
    xy_t sy;
    switch(sect){
        case 0:
            mapstx=HMAP_W;
            mapsty=topy;
            mapmaxx=MAP_W;
            mapmaxy=HMAP_H;
            sy=0;
            break;
        case 1:
            mapstx=HMAP_W;
            mapsty=HMAP_H;
            mapmaxx=MAP_W;
            mapmaxy=MAP_H;
            sy=HMAP_H-topy;
            break;
        case 2:
            mapstx=0;
            mapsty=HMAP_H;
            mapmaxx=HMAP_W;
            mapmaxy=MAP_H;
            sy=HMAP_H-topy;
            break;
        case 3:
            mapstx=0;
            mapsty=topy;
            mapmaxx=HMAP_W;
            mapmaxy=HMAP_H;
            sy=0;
            break;
    }
    for(y=mapsty;y<mapmaxy;y++,sy++){
        wattrset(game_win,COLOR_PAIR(ICON_FLOOR));
        mvwaddstr(game_win,sy,0,"    ");
        mvwaddstr(game_win,sy,MAP_W+4,"    ");
        for(x=mapstx;x<mapmaxx;x++){
            if(show){
                switch((icon=map->buf[y][x])){
                    case ICON_WALL:
                        wattrset(game_win,COLOR_PAIR(ICON_XXX+1));
                        mvwaddch(game_win,sy,4+x,icon_to_mapchar(icon));
                        break;
                    case ICON_MASK:
                    case ICON_EXIT:
                        wattrset(game_win,COLOR_PAIR(icon));
                        mvwaddch(game_win,sy,4+x,icon_to_mapchar(icon));
                        break;
                    default:
                        wattrset(game_win,COLOR_PAIR(ICON_FLOOR));
                        mvwaddch(game_win,sy,4+x,icon_to_mapchar(ICON_FLOOR));
                        break;
                }
            }
            else {
                wattrset(game_win,COLOR_PAIR(0));
                mvwaddch(game_win,sy,4+x,' ');
            }
        }
    }
*/
}

void game_win_dump_map(xy_t topy)
{
/*
    su_t i;
    su_t mb=1;
    for(i=0;i<4;i++,mb*=2)
        game_win_dump_map_sect(topy,i,mb&player.have_map);
    game_win_refresh(game_win);
    return;
*/
}

void game_win_map_display()
{
/*
    int key;
    xy_t y=player.map_view_y;
    xy_t maxscrolly=MAP_W-ICON_H*game_win->blk_count_y;
    game_win_dump_map(y);
    while(1){
        key=wgetch(game_win);
        if(key!=ERR){
            switch(key){
                case '\'':
                case KEY_UP:
                    if(y>0) {
                        y--;
                        game_win_dump_map(y);
                    }
                    break;
                case '/':
                case KEY_DOWN:
                    if(y<maxscrolly){
                        y++;
                        game_win_dump_map(y);
                    }
                    break;
                case 'm':
                case 'M':
                    game_win_display(y);
                    player.map_view_y=y;
                    return;
            }
        }
    }
*/
}
