/*
    XorGramana Copyright 2009 James W. Morris, james@jwm-art.net

    This file is part of XorGramana.

    XorGramana is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    XorGramana is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with XorGramana.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "movelist.h"

#include "map.h"

#include <stdlib.h>

struct xmv_list* xmvlist=0;

struct xmv_list* xmvlist_create()
{
    if(xmvlist)
        xmvlist_destroy();
    if(!(xmvlist=malloc(sizeof(struct xmv_list))))
        return 0;
    xmvlist->first=xmvlist->last=xmvlist->current=0;
    #ifdef MOVE_DEBUG
    printf("created xmvlist=%lx\n",(unsigned long)xmvlist);
    #endif
    return xmvlist;
}

void xmvlist_destroy()
{
    struct xmv_link* tmp=0;
    xmvlist->current=xmvlist->first;
    while(xmvlist->current){
        tmp=xmvlist->current;
        xmvlist->current=tmp->next;
        #ifdef MOVE_DEBUG
        printf("destroyed xmv_link %lx\n",(unsigned long)tmp);
        #endif
        free(tmp);
    }
    #ifdef MOVE_DEBUG
    printf("destroyed xmvlist %lx\n",(unsigned long)xmvlist);
    #endif
    free(xmvlist);
    xmvlist=0;
}

struct xmv_link* xmvlist_first()
{
    return xmvlist->current=xmvlist->first;
}

struct xmv_link* xmvlist_last()
{
    return xmvlist->current=xmvlist->first;
}

struct xmv_link* xmvlist_prev()
{
    if(xmvlist->current)
        if(xmvlist->current->prev)
            return (xmvlist->current=xmvlist->current->prev);
    return xmvlist->current=0;
}

struct xmv_link* xmvlist_next()
{
    if(xmvlist->current)
        if(xmvlist->current->next)
            return (xmvlist->current=xmvlist->current->next);
    return xmvlist->current=0;
}

struct xmv_link* xmvlist_cycle_prev()
{
    if(xmvlist->current){
        if(xmvlist->current==xmvlist->first)
            return (xmvlist->current=xmvlist->last);
        else
            return (xmvlist->current=xmvlist->current->prev);
    }
    return 0;
}

struct xmv_link* xmvlist_cycle_next()
{
    if(xmvlist->current){
        if(xmvlist->current==xmvlist->last)
            return (xmvlist->current=xmvlist->first);
        else
            return (xmvlist->current=xmvlist->current->next);
    }
    return 0;
}

struct xmv_link* xmvlist_current()
{
    return xmvlist->current;
}

struct xmv_link* xmvlist_append_xor_move(struct xor_move* xmv)
{
    struct xmv_link* lnk=0;
    if(!(lnk=malloc(sizeof(struct xmv_link))))
        return 0;
    lnk->xmv=xmv;
    lnk->next=0;
    if(!xmvlist->first){
        xmvlist->first=xmvlist->last=xmvlist->current=lnk;
        lnk->prev=0;
        #ifdef MOVE_DEBUG
        printf("created xmv_link %lx for xor_move %lx\n",
            (unsigned long)lnk, (unsigned long)xmv);
        #endif
        return lnk;
    }
    xmvlist->last->next=lnk;
    lnk->prev=xmvlist->last;
    xmvlist->last=lnk;
    #ifdef MOVE_DEBUG
    printf("created xmv_link %lx for xor_move %lx\n",
        (unsigned long)lnk, (unsigned long)xmv);
    #endif
    return lnk;
}

struct xor_move* xmvlist_unlink_xor_move(struct xmv_link* lnk)
{
    struct xmv_link* tmp=0;
    struct xor_move* xmv=lnk->xmv;
    #ifdef MOVE_DEBUG
    printf("xor_move* xmvlist_unlink_xor_move(xmv_link* lnk=%lx)\n",
        (unsigned long)lnk);
    if(lnk==xmvlist->current)
        printf("\tunlinking xmvlist->current\n");
    #endif
    if(lnk==xmvlist->first){
        if(xmvlist->first==xmvlist->last){
            xmvlist->first=xmvlist->last=xmvlist->current=0;
            #ifdef MOVE_DEBUG
            printf("destroyed xmv_link (first/last) %lx for xor_move %lx\n",
                (unsigned long)lnk, (unsigned long)xmv);
            #endif
            free(lnk);
            return xmv;
        }
        if(xmvlist->current==lnk)
            xmvlist->current=lnk->next;
        xmvlist->first=lnk->next;
        xmvlist->first->prev=0;
        #ifdef MOVE_DEBUG
        printf("destroyed xmv_link (first) %lx for xor_move %lx\n",
            (unsigned long)lnk, (unsigned long)xmv);
        #endif
        free(lnk);
        return xmv;
    }
    if(lnk==xmvlist->last){
        if(xmvlist->current==lnk)
            xmvlist->current=xmvlist->first;
        xmvlist->last=lnk->prev;
        xmvlist->last->next=0;
        #ifdef MOVE_DEBUG
        printf("destroyed xmv_link (last) %lx for xor_move %lx\n",
            (unsigned long)lnk, (unsigned long)xmv);
        #endif
        free(lnk);
        return xmv;
    }
    tmp=lnk->next;
    if(xmvlist->current==lnk)
        xmvlist->current=tmp;
    tmp->prev=lnk->prev;
    tmp=lnk->prev;
    tmp->next=lnk->next;
    #ifdef MOVE_DEBUG
    printf("destroyed xmv_link %lx for xor_move %lx\n",
        (unsigned long)lnk, (unsigned long)xmv);
    #endif
    free(lnk);
    return xmv;
}

struct xmv_link* xmvlist_contains_coord
(
    xy_t x,
    xy_t y,
    struct xmv_link* stop_at,
    struct xor_move** res_prev
)
{
    struct xmv_link* tmp=xmvlist->current;
    struct xmv_link* result=0;
    struct xor_move* xmv;
    struct xor_move* rpxmv;
    xmvlist_first();
    while((result=xmvlist->current)&&xmvlist->current!=stop_at){
        rpxmv=0;
        xmv=xmvlist->current->xmv;
        switch(xmv->dir){
            case MV_LEFT:
                if(xmv->from_y==y&&xmv->from_x<=x){
                    while(xmv){
                        if(xmv->from_x==x){
                            xmvlist->current=tmp;
                            if(res_prev)
                                *res_prev=rpxmv;
                            return result;
                        }
                        rpxmv=xmv;
                        xmv=xmv->right;
                    }
                }
                break;
            case MV_RIGHT:
                if(xmv->from_y==y&&xmv->from_x>=x){
                    while(xmv){
                        if(xmv->from_x==x){
                            xmvlist->current=tmp;
                            if(res_prev)
                                *res_prev=rpxmv;
                            return result;
                        }
                        rpxmv=xmv;
                        xmv=xmv->left;
                    }
                }
                break;
            case MV_DOWN:
                if(xmv->from_x==x&&xmv->from_y<=y){
                    while(xmv){
                        if(xmv->from_y==y){
                            xmvlist->current=tmp;
                            if(res_prev)
                                *res_prev=rpxmv;
                            return result;
                        }
                        rpxmv=xmv;
                        xmv=xmv->up;
                    }
                }
                break;
            case MV_UP:
                if(xmv->from_x==x&&xmv->from_y>=y){
                    while(xmv){
                        if(xmv->from_y==y){
                            xmvlist->current=tmp;
                            if(res_prev)
                                *res_prev=rpxmv;
                            return result;
                        }
                        rpxmv=xmv;
                        xmv=xmv->down;
                    }
                }
                break;
            default:
                break;
        }
        xmvlist_next();
    }
    if(res_prev)
        res_prev=0;
    xmvlist->current=tmp;
    return 0;
}
