/*
    XorGramana Copyright 2009 James W. Morris, james@jwm-art.net

    This file is part of XorGramana.

    XorGramana is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    XorGramana is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with XorGramana.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _MOVELIST_H
#define _MOVELIST_H

#include "actions.h"
#include "types.h"

struct xmv_link
{
    struct xor_move* xmv;
    struct xmv_link* prev;
    struct xmv_link* next;
};

struct xmv_list
{
    struct xmv_link* first;
    struct xmv_link* last;
    struct xmv_link* current;
};

extern struct xmv_list* xmvlist;

/* basic list functionality */
struct xmv_list* xmvlist_create();
void xmvlist_destroy();

struct xmv_link* xmvlist_first();
struct xmv_link* xmvlist_last();
struct xmv_link* xmvlist_prev();
struct xmv_link* xmvlist_next();

struct xmv_link* xmvlist_cycle_prev();
struct xmv_link* xmvlist_cycle_next();

struct xmv_link* xmvlist_append_xor_move(struct xor_move* xmv);
struct xor_move* xmvlist_unlink_xor_move(struct xmv_link* lnk);


/* xor_move relevant functions */

/*
    xmvlist_contains_coord searches the xmvlist for xor_moves
    satisfying (from_x == x && from_y == y). the function restores
    xmv_list->current to the value it had before xmvlist_contains_coord
    was called.

    when the link stop_at is encountered the search is abandonded.

    the xor_move* res_prev pointer is set to point, when a result is
    found, to the xor_move preceding the result in the chain, or 0.
    (for purposes of splitting the chain)
*/
struct xmv_link* xmvlist_contains_coord(
    xy_t x,
    xy_t y,
    struct xmv_link* stop_at,
    struct xor_move** res_prev);


#endif
