/*
    XorGramana Copyright 2009 James W. Morris, james@jwm-art.net

    This file is part of XorGramana.

    XorGramana is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    XorGramana is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with XorGramana.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "moves.h"
#include "game_display.h"
#include "explode.h"
#include "options.h"
#include "map.h"

#include <stdlib.h>

void init_unsupported()
{
    xy_t x,y;
    xy_t nx,ny;
    su_t i,ni;
    su_t dir;
    struct xor_move* xmv;
    #ifdef MOVE_DEBUG
    printf("init_unsupported()/*check for unsupported gravitating objects*/\n");
    #endif
    xmvlist_create();
    for(y=1;y<map->height;y++){
        for(x=1;x<map->width;x++){
            i=map->data[y][x];
            if(actions[i].mvini==MVI_GRAVITY){
                nx=0;ny=0;
                switch((dir=actions[i].mvi_dir)){
                    case MV_UP:
                        nx=x;ny=y-1;
                        break;
                    case MV_DOWN:
                        nx=x;ny=y+1;
                        break;
                    case MV_LEFT:
                        nx=x-1;ny=y;
                        break;
                    case MV_RIGHT:
                        nx=x+1;ny=y;
                        break;
                    default:
                        break;
                }
                if(nx>0&&ny>0&&nx<map->width&&ny<map->height){
                    ni=map->data[ny][nx];
                    xmv=create_xor_move(x,y,dir);
                    if(move_object_init(xmv)==CT_PASS){
                        if((create_gravity_chain(xmv))){
                            xmvlist_append_xor_move(xmv);
                            #ifdef MAP_DEBUG
                            printf("reporting:Unsupported object in map x:%d y:%d char:%c\n",
                                x,y,icon_details[xmv->from_obj].mapchar);
                            #endif
                        }
                        else
                            free(xmv);
                    }
                    else free(xmv);
                }
            }
        }
    }
    if(xmvlist_first()){
        if(!player.replay){
            gfx_message("Get Ready");
            game_win_display();
        }
        move_gravity_process();
    }
    xmvlist_destroy();
}

ct_t move_object_init(struct xor_move* move)
{
    ct_t cont;
    move->to_x=move->from_x;
    move->to_y=move->from_y;
    #ifdef MOVE_DEBUG
    printf("move_object_init(xor_move* %lx)\n",(unsigned long)move);
    #endif
    switch(move->dir){
        case MV_LEFT:   move->to_x--;   break;
        case MV_RIGHT:  move->to_x++;   break;
        case MV_UP:     move->to_y--;   break;
        case MV_DOWN:   move->to_y++;   break;
        default:
            fprintf(stderr,
                "Invalid movement direction error! move:%d\n",move->dir);
            return CT_BLOCK;
    }
    move->from_obj=map->data[move->from_y][move->from_x];
    move->to_obj=map->data[move->to_y][move->to_x];
    cont=actions[move->to_obj].cont;
    /* ------------------------------------------ */
    /* conditions apply both to player and object */
    if(cont==CT_BLOCK||cont==CT_PASS)
        return cont;
    if(actions[move->from_obj].cont&CT_HARDPUSH){
        if(move->dir&actions[move->from_obj].cont_dir){
            /*fprintf(stderr,"hardpush causing blockage..\n");*/
            return CT_BLOCK;
        }
    }
    if(cont==CT_FILTER){
        if(actions[move->to_obj].cont_dir&move->dir)
            return CT_PASS;
        return CT_BLOCK;
    }
    /* -------------------------------- */
    /* conditions which apply to player */
    if(actions[move->from_obj].mvini&MVI_PLAYER){
        if(cont&CT_PUSH){
            if(actions[move->to_obj].cont_dir&move->dir)
                return CT_PUSH;
            else
                return CT_BLOCK;
        }
        if(cont&CT_PICKUP||cont&CT_TELEPORT||cont&CT_EXIT)
            return cont;
        if(cont&CT_HARDPUSH){
            if(actions[move->to_obj].cont_dir&move->dir)
                return CT_HARDPUSH;
            else
                return CT_BLOCK;
        }
        if(cont&CT_PUSH){
            if(actions[move->to_obj].cont_dir&move->dir)
                return CT_PUSH;
            else
                return CT_BLOCK;
        }
        return CT_ERROR;
    }
    /* -------------------------------- */
    /* conditions which apply to object */
    if(cont&CT_EXPLODE)
        return CT_EXPLODE;
    if(cont&CT_FLIP)
        return CT_FLIP;
    return CT_BLOCK;
}

void move_gravity_process()
{
    struct xor_move* xmv=0;
    struct xor_move* tmp=0;
    struct xor_move* cmv=0;
    struct xor_move* cmv_next=0;
    struct xmv_link* tmplnk=0;
    ct_t cont;
    xy_t tmpx,tmpy;
    unsigned int flimit=SDL_GetTicks()+TICK_COUNT;
    #ifdef MOVE_DEBUG
    printf("\nmove_gravity_process()\n");
    #endif
    /*
    rpause.tv_sec=0;
    if(player.replay)
        rpause.tv_nsec=options_replay_speed(options->replay_speed);
    else
        rpause.tv_nsec=20000000L;
    */
    xmvlist_first();
    while(xmvlist->current){
        xmv=xmvlist->current->xmv;
        cmv=xmv;
        #ifdef MOVE_DEBUG
        printf("xmvlist->current=%lx->xmv=%lx\n",
            (unsigned long)xmvlist->current,(unsigned long)xmv);
        #endif
        while(cmv){
            delay(flimit);
            flimit=SDL_GetTicks()+TICK_COUNT;
            #ifdef MOVE_DEBUG
            printf("mv_grv_proc:\n\t*(%lx)cmv->from_x:%d,from_y:%d,from_obj:%s\n",
                (unsigned long)cmv,cmv->from_x,cmv->from_y,
                icon_details[cmv->from_obj].fname);
            #endif
            switch(cmv->dir){
                case MV_UP:     cmv_next=cmv->down; break;
                case MV_DOWN:   cmv_next=cmv->up;   break;
                case MV_LEFT:   cmv_next=cmv->right;break;
                case MV_RIGHT:  cmv_next=cmv->left; break;
                default:
                    break;
            }
            cont=move_object_init(cmv);
            if(actions[cmv->to_obj].mvini==MVI_PLAYER){
                if(cmv->moves_count>0){
                    player_death(cmv->to_obj);
                    cont=CT_PASS;
                }
            }
            if(cont&CT_FLIP){
                if(cmv->moves_count>0){
                    #ifdef MOVE_DEBUG
                    printf("grav proc flipping..\n");
                    #endif
                    game_win_icon_display(
                        cmv->to_x,cmv->to_y,
                        (   map->data[cmv->to_y][cmv->to_x]=
                            flip(cmv->to_obj,cmv->dir)  )      );
                    cont=CT_BLOCK;
                }
            }
            if(cont&CT_EXPLODE){
                #ifdef MOVE_DEBUG
                printf("\nBANG!!!\n");
                #endif
                explode_process_detonator(xmvlist->current);
                cmv=cmv_next;
                #ifdef MOVE_DEBUG
                printf("\nreturn to move_gravity_process(xor_move* xmv=%lx)\n",(unsigned long) xmv);
                printf("\tcmv=cmv_next=%lx\n",(unsigned long)cmv);
                if(cmv)
                    printf("\t*(%lx)cmv->from_x:%d,from_y:%d,from_obj:%s\n",
                        (unsigned long)cmv,cmv->from_x,cmv->from_y,
                            icon_details[cmv->from_obj].fname);
                #endif
            }
            else if(cont==CT_PASS)
            {
                map->data[cmv->from_y][cmv->from_x]=ICON_FLOOR;
                map->data[cmv->to_y][cmv->to_x]=cmv->from_obj;
                game_win_move_object(cmv);
                tmpx=cmv->from_x;
                tmpy=cmv->from_y;
                cmv->from_x=cmv->to_x;
                cmv->from_y=cmv->to_y;
                cmv->moves_count++;
                if(cmv_next){
                    cmv=cmv_next;
                    /*rpause.tv_nsec/=1.1;*/
                }
                else{
                    if(cmv->dir&MV_HORIZ){
                        if((cmv=create_gravity_chain_xydir(tmpx,tmpy-1,MV_DOWN)))
                            xmvlist_append_xor_move(cmv);
                        if((tmp=create_gravity_chain_xydir(tmpx,tmpy+1,MV_UP))){
                            xmvlist_append_xor_move(tmp);
                            if(!cmv)
                                cmv=tmp;
                        }
                    }
                    else{
                        if((cmv=create_gravity_chain_xydir(tmpx+1,tmpy,MV_LEFT)))
                            xmvlist_append_xor_move(cmv);
                        if((tmp=create_gravity_chain_xydir(tmpx-1,tmpy,MV_RIGHT))){
                            xmvlist_append_xor_move(tmp);
                            if(!cmv)
                                cmv=tmp;
                        }
                    }
                    /*
                    if(cmv){
                        rpause.tv_nsec/=1.1;
                    }
                    */
                    xmvlist_cycle_next();
                    cmv=xmv=xmvlist->current->xmv;
                }
            }
            else if ((cmv=move_unchain_blocked_bomb(xmvlist->current)))
                xmv=cmv;
            else { /* chain blocked - remove it */
                #ifdef MOVE_DEBUG
                printf("mv_grav_proc() chain blocked removing...\n");
                #endif
                tmplnk=xmvlist->current;
                #ifdef MOVE_DEBUG
                if(tmplnk->xmv!=xmv)
                    printf("\n***** current->xmv != xmv *******\n");
                #endif
                destroy_gravity_chain(xmv);
                if(xmvlist_cycle_next()==tmplnk)
                    cmv=xmv=0; /* about to remove last chain */
                else
                    cmv=xmv=xmvlist->current->xmv;
                xmvlist_unlink_xor_move(tmplnk);
            }
        }
    }
    #ifdef MOVE_DEBUG
    printf("\nEXITING move_gravity_process(xor_move* xmv=%lx)\n",(unsigned long) xmv);
    #endif
}

struct xor_move* move_unchain_blocked_bomb(struct xmv_link* lnk)
{
    struct xor_move* xmv=lnk->xmv;
    struct xor_move* tmp;
    if(xmv->moves_count==0)
        return 0;
    #ifdef MOVE_DEBUG
    printf("move_unchain_blocked_bomb(xmv_link* lnk=%lx)\n",
        (unsigned long)lnk);
    printf("moves count:%d\n",xmv->moves_count);
    #endif
    switch(xmv->dir){
        case MV_DOWN:
            while(xmv){
                if(bomb_type(xmv->from_obj)==MV_HORIZ){
                    if((tmp=xmv->up)){
                        xmv->up=0;
                        destroy_gravity_chain(lnk->xmv);
                        return (lnk->xmv=tmp);
                    }
                    return 0;
                }
                else{
                    tmp=xmv;
                    xmv=xmv->up;
                }
            }
            return 0;
        case MV_LEFT:
            while(xmv){
                if(bomb_type(xmv->from_obj)==MV_VERT){
                    if((tmp=xmv->right)){
                        xmv->right=0;
                        destroy_gravity_chain(lnk->xmv);
                        return (lnk->xmv=tmp);
                    }
                    return 0;
                }
                else{
                    tmp=xmv;
                    xmv=xmv->right;
                }
            }
            return 0;
        case MV_UP:
            while(xmv){
                if(bomb_type(xmv->from_obj)==MV_HORIZ){
                    if((tmp=xmv->down)){
                        xmv->down=0;
                        destroy_gravity_chain(lnk->xmv);
                        return (lnk->xmv=tmp);
                    }
                    return 0;
                }
                else{
                    tmp=xmv;
                    xmv=xmv->down;
                }
            }
            return 0;
        case MV_RIGHT:
            while(xmv){
                if(bomb_type(xmv->from_obj)==MV_VERT){
                    if((tmp=xmv->right)){
                        xmv->left=0;
                        destroy_gravity_chain(lnk->xmv);
                        return (lnk->xmv=tmp);
                    }
                    return 0;
                }
                else{
                    tmp=xmv;
                    xmv=xmv->left;
                }
            }
            return 0;
        default:
            return 0;
    }
}

void move_hard_push(struct xor_move* xmv)
{
    #ifdef MOVE_DEBUG
    printf("move_hard_push (xor_move* %lx)\n",(unsigned long)xmv);
    #endif
    /*
    rpause.tv_sec=0;
    if(player.replay)
        rpause.tv_nsec=options_replay_speed(options->replay_speed);
    else
        rpause.tv_nsec=20000000L;
    */
    #ifdef MOVE_DEBUG
    if(!(actions[xmv->from_obj].cont_dir&xmv->dir))
        printf("***** hard push direction mismatch! *****\n");
    #endif
    unsigned int flimit=SDL_GetTicks()+TICK_COUNT;
    do{
        map->data[xmv->from_y][xmv->from_x]=ICON_FLOOR;
        map->data[xmv->to_y][xmv->to_x]=xmv->from_obj;
        game_win_move_object(xmv);
        xmv->from_x=xmv->to_x;
        xmv->from_y=xmv->to_y;
        switch(xmv->dir){
            case MV_LEFT:   xmv->to_x--;   break;
            case MV_RIGHT:  xmv->to_x++;   break;
            case MV_UP:     xmv->to_y--;   break;
            case MV_DOWN:   xmv->to_y++;   break;
            default:
                fprintf(stderr,
                    "***** hardpush object move non-direction *****\n");
                return;
        }
        delay(flimit);
        flimit=SDL_GetTicks()+TICK_COUNT;
        /*nanosleep(&rpause,&repause);*/
    }
    while(map->data[xmv->to_y][xmv->to_x]==ICON_FLOOR);

/*
    while(map->data[xmv->to_y][xmv->to_x]==ICON_FLOOR){
        map->data[xmv->from_y][xmv->from_x]=ICON_FLOOR;
        map->data[xmv->to_y][xmv->to_x]=xmv->from_obj;
        game_win_move_object(xmv);
        xmv->from_x=xmv->to_x;
        xmv->from_y=xmv->to_y;
        switch(xmv->dir){
            case MV_LEFT:   xmv->to_x--;   break;
            case MV_RIGHT:  xmv->to_x++;   break;
            case MV_UP:     xmv->to_y--;   break;
            case MV_DOWN:   xmv->to_y++;   break;
            default:
                fprintf(stderr,
                    "***** hardpush object move non-direction *****\n");
                return;
        }
        delay(flimit);
        flimit=SDL_GetTicks()+TICK_COUNT;
        */
        /*nanosleep(&rpause,&repause);*/
        /*
    }
*/
}

su_t flip(su_t obj, su_t dir)
{
    switch(obj){
        case ICON_B: return (dir&MV_VERT?ICON_P:ICON_D);
        case ICON_D: return (dir&MV_VERT?ICON_Q:ICON_B);
        case ICON_P: return (dir&MV_VERT?ICON_B:ICON_Q);
        case ICON_Q: return (dir&MV_VERT?ICON_D:ICON_P);
        case ICON_S: return ICON_Z;
        case ICON_Z: return ICON_S;
        case ICON_L: return ICON_J;
        case ICON_J: return ICON_L;
        default:
            fprintf(stderr,"tried to flip non-flippable\n");
            return obj;
    }
}
