/*
    XorGramana Copyright 2009 James W. Morris, james@jwm-art.net

    This file is part of XorGramana.

    XorGramana is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    XorGramana is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with XorGramana.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "options.h"

#include "map.h"

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

const char *map_dir="maps/";
const char *xormap_dir="xormaps/";
const char *data_dir=DATADIR;

char* map_name[MAX_LEVEL+1]={0};

struct xor_options* options=0;

/*
void options_init_user_data_dir();
*/

su_t options_create()
{
    char* tmp;
    if(!(options=malloc(sizeof(struct xor_options)))){
        printf("Memory allocation error!\n");
        exit(1);
    }
    options->game=0; /* 0=XorGramana, 1=Xor */
    options->show_indicators=1;
    options->scroll_thresh=3;
    options->good_opt_dir=0;
    options->replay_speed=7;
    options->dir_opt=0;
    options->data_dir=0;
    options->user_dir=0;
    options->map_dir=0;
    if((tmp=getenv("HOME"))){
        if((options->user_dir=malloc(strlen(tmp)+1)))
            strcpy(options->user_dir,tmp);
    }
    if(options_set_dir_opt(DATA_INST_LOC))
        return 1;
    if(options_set_dir_opt(DATA_CWD_LOC))
        return 1;
    options_destroy();
    return 0;
}

void options_destroy()
{
    if(options){
        if(options->map_dir)
            free(options->map_dir);
        if(options->user_dir)
            free(options->user_dir);
        free(options);
        options=0;
    }
}

su_t set_game(su_t n)
{
    FILE* fp;
    if(n==1)
        options->game=1;
    else
        options->game=0;
    if(options->map_dir)
        free(options->map_dir);
    if(!(options->map_dir=options_file_path(
            (options->game?xormap_dir:map_dir),
            options->data_dir)))
        return 0;
    if(!(fp=fopen(options->map_dir,"r"))){
        free(options->map_dir);
        options->map_dir=0;
        return 0;
    }
    fclose(fp);
    return options_create_map_names();
}

su_t options_set_dir_opt(enum DATA_LOC loc)
{
    switch((options->dir_opt=loc))
    {
        case DATA_CWD_LOC:  options->data_dir="data/";  break;
        default:            options->data_dir=data_dir; break;
    }
    return set_game(options->game);
}

su_t options_create_map_names()
{
    su_t i;
    for(i=MIN_LEVEL;i<=MAX_LEVEL;i++){
        if(map_name[i])
            free(map_name[i]);
        if(!(map_name[i]=xg_map_load_read_name(i)))
            if(i==MIN_LEVEL)
                return 0;
    }
    return 1;
}

void options_destroy_map_names()
{
    su_t i;
    for(i=MIN_LEVEL;i<=MAX_LEVEL;i++)
        free(map_name[i]);
}

long options_replay_speed(char n)
{
    if(n<1)
        n=1;
    if(n>9){
        if(n>='1'&&n<='9'){
            n=9-('9'-n);
        }
        else
        n=9;
    }
    options->replay_speed=n;
    return (long)(200000000L/(float)n);
}

char* options_map_filename(su_t level)
{
    char* fname;
    char* dir=0;
    su_t l;
    if(level<MIN_LEVEL||level>MAX_LEVEL)
        return 0;
    l=strlen(dir=options->map_dir);
    l+=((level<10?5:6)+1);
    if(!(fname=malloc(l)))
        return 0;
    if(snprintf(fname,l,"%s%d.txt",dir,level)!=l-1){
        free(fname);
        return 0;
    }
    return fname;
}

char* options_file_path(const char* fname, const char *path)
{
    char* fp=0;
    su_t n;
    if(!fname)
        return 0;
    if(path){
        if(path[n=strlen(path)]!='/')
            n++;
        if(!(fp=calloc(strlen(fname)+n+1,sizeof(char))))
            return 0;
        strcpy(fp,path);
        if(fp[n-1]!='/'){
            fp[n-1]='/';
            fp[n]=0;
        }
    }
    else{
        if(!(fp=calloc(strlen(fname)+1,sizeof(char))))
            return 0;
    }
    strcat(fp,fname);
    return fp;
}

