/*
    XorGramana Copyright 2009 James W. Morris, james@jwm-art.net

    This file is part of XorGramana.

    XorGramana is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    XorGramana is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with XorGramana.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _OPTIONS_H
#define _OPTIONS_H

#include "types.h"
#include "defs.h"

#include <stdlib.h>

/*  dir in this context = directory
    elsewhere, dir = direction...
*/

extern const char *map_dir;
extern const char *xormap_dir;
extern const char *data_dir;

enum DATA_LOC
{
    DATA_INST_LOC=  0,  /* installed location                   */
    DATA_CWD_LOC=   1,  /* current working directory location   */
    DATA_USER_LOC=  2   /* unused - user specified loc cmdline  */
};

extern char* map_name[MAX_LEVEL+1];

struct xor_options
{
    unsigned game           : 1; /* 0=XorGramana, 1=Xor */
    unsigned show_indicators: 1;
    unsigned scroll_thresh  : 2;
    unsigned good_opt_dir   : 1;
    unsigned replay_speed   : 4;
    unsigned dir_opt        : 2; /* 0=default, 1=cwd, 2=other */
    const char* data_dir;/* ie install dir where maps & help are. */
    char* user_dir; /* ie /home/username */
    char* map_dir;
};

extern struct xor_options* options;

su_t options_create();
void options_destroy();

su_t set_game(su_t n);

su_t options_set_dir_opt(enum DATA_LOC loc);

su_t options_create_map_names();
void options_destroy_map_names();

/*  options_replay_speed accepts either 1-9 or '1'-'9'  */
long options_replay_speed(char n);

char* options_map_filename(su_t level);
char* options_file_path(const char* fname, const char *path);
#endif
