/*
    XorGramana Copyright 2009 James W. Morris, james@jwm-art.net

    This file is part of XorGramana.

    XorGramana is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    XorGramana is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with XorGramana.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "play_xor.h"
#include <stdlib.h>
#include <time.h>

#include "moves.h"
#include "options.h"
#include "scores.h"
#include "map.h"
#include "icons.h"
#include "mapdisplay.h"

struct xg_key** play_keys=0;
struct xg_key** replay_keys=0;

bool play_create_keys()
{
    if(!(play_keys=create_key_list_array(PLAY_KEYS_XXX)))
        return FALSE;
    play_keys[PLAY_KEY_UP]->key=SDLK_UP;
    play_keys[PLAY_KEY_DOWN]->key=SDLK_DOWN;
    play_keys[PLAY_KEY_LEFT]->key=SDLK_LEFT;
    play_keys[PLAY_KEY_RIGHT]->key=SDLK_RIGHT;
    play_keys[PLAY_KEY_SWAP]->key=SDLK_RETURN;
    play_keys[PLAY_KEY_MAP]->key=SDLK_m;
    play_keys[PLAY_KEY_SCR_THRESH_1]->key=SDLK_1;
    play_keys[PLAY_KEY_SCR_THRESH_2]->key=SDLK_2;
    play_keys[PLAY_KEY_SCR_THRESH_3]->key=SDLK_3;
    play_keys[PLAY_KEY_TOG_INDICATORS]->key=SDLK_F1;
    #ifdef MAPDEVMODE
    play_keys[PLAY_KEY_MAPDEV]->key=SDLK_F9;
    #endif
    if(!(replay_keys=create_key_list_array(REPLAY_KEYS_XXX)))
        return FALSE;
    replay_keys[REPLAY_KEY_YES]->key=SDLK_y;
    replay_keys[REPLAY_KEY_NO]->key=SDLK_n;
    replay_keys[REPLAY_KEY_SAVE]->key=SDLK_s;
    replay_keys[REPLAY_KEY_CONTINUE]->key=SDLK_c;
    replay_keys[REPLAY_KEY_SPEED_1]->key=SDLK_1;
    replay_keys[REPLAY_KEY_SPEED_2]->key=SDLK_2;
    replay_keys[REPLAY_KEY_SPEED_3]->key=SDLK_3;
    replay_keys[REPLAY_KEY_SPEED_4]->key=SDLK_4;
    replay_keys[REPLAY_KEY_SPEED_5]->key=SDLK_5;
    replay_keys[REPLAY_KEY_SPEED_6]->key=SDLK_6;
    replay_keys[REPLAY_KEY_SPEED_7]->key=SDLK_7;
    replay_keys[REPLAY_KEY_SPEED_8]->key=SDLK_8;
    replay_keys[REPLAY_KEY_SPEED_9]->key=SDLK_9;
    return TRUE;
}

void play_destroy_keys()
{
    #ifdef INPUT_DEBUG
    printf("about to destroy play_keys\n");
    #endif
    destroy_key_list_array(play_keys);
    destroy_key_list_array(replay_keys);
}

bool play_xor(lvl_t level, bool from_start)
{
    ctr_t mv;
    su_t move;
    su_t state=PLAY_CONTINUE;
    bool map_solved;
    unsigned int flimit=SDL_GetTicks()+TICK_COUNT;
    player.replay=0;
    if(from_start){
        replay.level=level;
        init_wall(level,TRUE);
        xg_map_load(level);
        if(!map)
            return FALSE;
        xg_map_apply_edging();
        map_display_init();
        if((replay.moves=malloc(sizeof(su_t)*map->max_moves+1))){
            for(mv=map->max_moves;mv>0;mv--)
                replay.moves[mv]=0;
        }
        else
            return FALSE;
        xg_map_init_views(game_win->blk_count_x,game_win->blk_count_y);
        player_init();
    }
    else
        level=replay.level;
    gfx_set_win(scr_win);
    gfx_win_half_size();
    set_space_icon(SPC_TRAN);
    gfx_mv_cur(2,0);
    gfx_printf("%s",map->title);
    gfx_set_win(game_win);
    gfx_win_full_size();
    game_win_display();
    set_keys(play_keys);
    if(from_start)
        init_unsupported();
    game_win_update_player_info();
    map_solved=map_check_solved();
    while(state==PLAY_CONTINUE||state==PLAY_PROCESS_MOVE){
        move=MV_NONE;
        if(simple_poll_event()){
            if(xginput.exit)
                return FALSE;
            else if(xginput.quit){
                dump_replay_data();
                move=MV_PLAYER_QUIT;
            }
            else if(play_keys[PLAY_KEY_LEFT]->pressed)  move=MV_LEFT;
            else if(play_keys[PLAY_KEY_RIGHT]->pressed) move=MV_RIGHT;
            else if(play_keys[PLAY_KEY_UP]->pressed)    move=MV_UP;
            else if(play_keys[PLAY_KEY_DOWN]->pressed)  move=MV_DOWN;
            else if(play_keys[PLAY_KEY_SWAP]->pressed)  move=MV_PLAYER_SWAP;
            else if(play_keys[PLAY_KEY_MAP]->pressed)   map_display_show();
            else if(play_keys[PLAY_KEY_SCR_THRESH_1]->pressed)
                options->scroll_thresh=1;
            else if(play_keys[PLAY_KEY_SCR_THRESH_2]->pressed)
                options->scroll_thresh=2;
            else if(play_keys[PLAY_KEY_SCR_THRESH_3]->pressed)
                options->scroll_thresh=3;
            else if(play_keys[PLAY_KEY_TOG_INDICATORS]->pressed){
                options->show_indicators^=1;
                game_win_display();
            }
            #ifdef MAPDEVMODE
            else if(play_keys[PLAY_KEY_MAPDEV]->pressed){
                map_dump_solve_state();
            }
            #endif
            if(move){
                if((state=player_move(move))&PLAY_RECORD){
                    replay.moves[player.move_no++]=move;
                    state^=PLAY_RECORD;
                    if(state==PLAY_ZERO_MOVES)
                        gfx_message("out of moves");
                    if(map_check_solved()!=map_solved){
                        map_solved^=1;
                        game_win_icon_display(
                            map->exit.x,
                            map->exit.y,
                            map->data[map->exit.y][map->exit.x]);
                    }
                    #ifdef MOVE_DEBUG
                    printf("player x:%d y:%d\n",
                        map->player[player.player].x,
                        map->player[player.player].y);
                    #endif
                    if((!player.p0_alive)&&(!player.p1_alive))
                        state=PLAY_GOTCHA;
                    game_win_update_player_info();
                }
            }
        }
        delay(flimit);
        flimit=SDL_GetTicks()+TICK_COUNT;
    }
    if(state==PLAY_COMPLETE){
        save_score(map->level,player.move_no);
        gfx_message("well done");
    }
    dump_replay_data();
    if(player.move_no<map->max_moves)
        return replay_xor(TRUE);
    return FALSE;
}

void dump_replay_data()
{
    int n;
    char c;
    return;
    if(!replay.moves){
        printf("no replay data!\n");
        return;
    }
    printf("moves remaining:%d\n", map->max_moves-player.move_no);
    for(n=0;replay.moves[n]!=MV_NONE;n++){
        switch(replay.moves[n]){
            case MV_UP:             c='u';break;
            case MV_DOWN:           c='d';break;
            case MV_LEFT:           c='l';break;
            case MV_RIGHT:          c='r';break;
            case MV_HORIZ:          c='h';break;
            case MV_VERT:           c='v';break;
            case MV_ANY:            c='a';break;
            case MV_PLAYER_SWAP:    c='s';break;
            case MV_PLAYER_QUIT:    c='q';break;
        }
        printf("%d:%c ",n,c);
    }
    printf("\n");
}

void dump_play_state(int mv, int state)
{
    printf("%d:state( ",mv);
    if(state&PLAY_RECORD){
        printf("record ");
        state^=PLAY_RECORD;
    }
    switch(state){
        case PLAY_CONTINUE:
            printf("continue ");
            break;
        case PLAY_PROCESS_MOVE:
            printf("process ");
            break;
        case PLAY_SWAP:
            printf("swap ");
            break;
        case PLAY_QUIT:
            printf("quit ");
            break;
        case PLAY_ZERO_MOVES:
            printf("zero moves");
            break;
        case PLAY_GOTCHA:
            printf("gotcha ");
            break;
        case PLAY_COMPLETE:
            printf("complete ");
            break;
        default:
            printf("*** UNKNOWN **** ");
            break;
    }
    printf(") ");
}

bool replay_xor(bool ask)
{
    unsigned int flimit=SDL_GetTicks()+TICK_COUNT;
    su_t state;
    int p;
    bool continue_play=FALSE;
    bool ask_continue=player.p0_alive|player.p1_alive;
    if(!replay.moves)
        return FALSE;
    dump_replay_data();
    set_keys(replay_keys);
    player.replay=TRUE;
    do{
        if(ask==TRUE){
            if(ask_continue)
                gfx_message("Replay (Y/N/C/S)?");
            else
                gfx_message("Replay (Y/N/S)?");
            if(xginput.exit||replay_keys[REPLAY_KEY_NO]->pressed)
            {
                free(replay.moves);
                replay.moves=0;
                xg_map_destroy();
                return FALSE;
            }
            else if(replay_keys[REPLAY_KEY_SAVE]->pressed){
                replay_save();
                free(replay.moves);
                replay.moves=0;
                xg_map_destroy();
                return FALSE;
            }
            else if(replay_keys[REPLAY_KEY_CONTINUE]->pressed){
                if(ask_continue)
                    if(player.p0_alive||player.p1_alive)
                        continue_play=TRUE;
            }
        }
        else{
            ask=TRUE;
            replay_keys[REPLAY_KEY_YES]->pressed=TRUE;
            /*SDL_PushEvent(&fake_confirm);*/
        }
        if(replay_keys[REPLAY_KEY_YES]->pressed){
            dump_replay_data();
            xg_map_load(replay.level);
            xg_map_apply_edging();
            xg_map_init_views(game_win->blk_count_x,game_win->blk_count_y);
            player_init();
            init_wall(replay.level,TRUE);
            game_win_display();
            state=PLAY_CONTINUE;
            init_unsupported();
            dump_replay_data();
            while(state==PLAY_CONTINUE||state==PLAY_PROCESS_MOVE)
            {
                if(simple_poll_event()){
                    if(xginput.exit||xginput.quit)
                        state=player_move(MV_PLAYER_QUIT);
                    else if(replay_keys[REPLAY_KEY_SPEED_1]->pressed)
                        options->replay_speed=1;
                    else if(replay_keys[REPLAY_KEY_SPEED_2]->pressed)
                        options->replay_speed=2;
                    else if(replay_keys[REPLAY_KEY_SPEED_3]->pressed)
                        options->replay_speed=3;
                    else if(replay_keys[REPLAY_KEY_SPEED_4]->pressed)
                        options->replay_speed=4;
                    else if(replay_keys[REPLAY_KEY_SPEED_5]->pressed)
                        options->replay_speed=5;
                    else if(replay_keys[REPLAY_KEY_SPEED_6]->pressed)
                        options->replay_speed=6;
                    else if(replay_keys[REPLAY_KEY_SPEED_7]->pressed)
                        options->replay_speed=7;
                    else if(replay_keys[REPLAY_KEY_SPEED_8]->pressed)
                        options->replay_speed=8;
                    else if(replay_keys[REPLAY_KEY_SPEED_9]->pressed)
                        options->replay_speed=9;
                }
                else {
                    state=player_move(
                        replay.moves[player.move_no++])
                            ^PLAY_RECORD;
                    map_check_solved();
                    game_win_update_player_info();
                    if((!player.p0_alive)&&(!player.p1_alive))
                        state=PLAY_GOTCHA;
                    for(p=0;p<options->replay_speed;p++){
                        delay(flimit);
                        flimit=SDL_GetTicks()+TICK_COUNT;
                    }
                }
            }
            if(state==PLAY_GOTCHA)
                ask_continue=FALSE;
            else
                ask_continue=TRUE;
        }
    }while(!continue_play);
    state=PLAY_CONTINUE;
    if(replay.moves[player.move_no-1]==MV_PLAYER_QUIT)
        player.move_no--;
    return TRUE;
}

void replay_save()
{
/*
    char buf[32];
    FILE* fp;
    int key;
    echo();
    wattrset(game_win,COLOR_PAIR(ICON_XXX+1));
    mvwprintw(game_win,11,0,"                                        ");
    mvwprintw(game_win,12,0," Filename:                              ");
    mvwprintw(game_win,13,0,"                                        ");
    wmove(game_win,12,10);
    wgetnstr(game_win,buf,30);
    noecho();
    if((fp=fopen(buf,"r"))){
        mvwprintw(game_win,15,0,"                          ");
        mvwprintw(game_win,16,0," Exists! Overwrite (y/n)? ");
        mvwprintw(game_win,17,0,"                          ");
        key=wgetch(game_win);
        if(key=='n'||key=='N')
            return;
        fclose(fp);
    }
    if(!(fp=fopen(buf,"w"))){
        mvwprintw(game_win,18,0,"        ");
        mvwprintw(game_win,19,0," Error! ");
        mvwprintw(game_win,20,0,"        ");
        wgetch(game_win);
        return;
    }
    if(!fwrite(&replay,sizeof(struct xor_replay),1,fp))
    {
        fclose(fp);
        mvwprintw(game_win,18,0,"        ");
        mvwprintw(game_win,19,0," Error! ");
        mvwprintw(game_win,20,0,"        ");
        wgetch(game_win);
        return;
    }
    fclose(fp);
    mvwprintw(game_win,18,0,"        ");
    mvwprintw(game_win,19,0," Saved! ");
    mvwprintw(game_win,20,0,"        ");
    wgetch(game_win);
*/
}


lvl_t replay_load()
{
    return 0;
/*
    char buf[32];
    FILE* fp;
    echo();
    wattrset(game_win,COLOR_PAIR(ICON_XXX+1));
    mvwprintw(game_win,11,0,"                                        ");
    mvwprintw(game_win,12,0," Filename:                              ");
    mvwprintw(game_win,13,0,"                                        ");
    wmove(game_win,12,10);
    wgetnstr(game_win,buf,30);
    noecho();
    if(!(fp=fopen(buf,"r"))){
        mvwprintw(game_win,18,0,"        ");
        mvwprintw(game_win,19,0," Error! ");
        mvwprintw(game_win,20,0,"        ");
        wgetch(game_win);
        return -1;
    }
    if(!fread(&replay,sizeof(struct xor_replay),1,fp))
    {
        fclose(fp);
        mvwprintw(game_win,18,0,"        ");
        mvwprintw(game_win,19,0," Error! ");
        mvwprintw(game_win,20,0,"        ");
        wgetch(game_win);
        return -1;
    }
    mvwprintw(game_win,18,0,"         ");
    mvwprintw(game_win,19,0," Loaded! ");
    mvwprintw(game_win,20,0,"         ");
    if(replay_xor(FALSE))
        return 100;
    return replay.level;
*/
}

void inc_max_moves()
{
    su_t* tmp=0;
    if(player.replay){
        /* really don't want to realloc during replay */
        map->max_moves+=10;
        return;
    }
    if((tmp=realloc(replay.moves,map->max_moves+10))){
        map->max_moves+=10;
        replay.moves=tmp;
    }
    else{
        fprintf(stderr,"*** memory re-allocation for replay data failed! ***\n");
        fprintf(stderr,"This was needed for the object just collected by player\n");
        fprintf(stderr,"in order to increase the number of moves allowed in which\n");
        fprintf(stderr,"to complete this level. This therefor is staying the same.\n");
    }
}
