/*
    XorGramana Copyright 2009 James W. Morris, james@jwm-art.net

    This file is part of XorGramana.

    XorGramana is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    XorGramana is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with XorGramana.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "player.h"
#include "moves.h"
#include "game_display.h"
#include "actions.h"
#include "moves.h"
#include "movelist.h"
#include "teleport.h"
#include "play_xor.h"
#include "options.h"

#include <stdlib.h>

void player_init()
{
    player.move_no=0;
    player.masks_collected=0;
    player.have_map=0;
    player.p0_alive=1;
    player.p1_alive=1;
    player.player=0;
    player.wall_vis=TRUE;
}

enum PLAY_STATE player_move(su_t move)
{
    struct xor_move* pmv;
    struct xor_move* omv;
    ct_t cont;
    if(player.move_no==map->max_moves)
        return PLAY_ZERO_MOVES|PLAY_RECORD;
    if(move==MV_PLAYER_QUIT)
        return (player.move_no==map->max_moves)
            ?PLAY_QUIT
            :PLAY_QUIT|PLAY_RECORD;
    if(move==MV_PLAYER_SWAP){
        if((player.player?player.p0_alive:player.p1_alive)){
            player.player^=1;
            game_win_swap_update();
            game_win_display();
            return PLAY_CONTINUE|PLAY_RECORD;
        }
        return PLAY_CONTINUE;
    }
    pmv=create_xor_move(map->player[player.player].x,
                        map->player[player.player].y, move);
    cont=move_object_init(pmv);
    switch(cont)
    {
        case CT_BLOCK:
            free(pmv);
            return PLAY_CONTINUE;
        case CT_EXIT:
            free(pmv);
            return PLAY_COMPLETE|PLAY_RECORD;
        case CT_TELEPORT:
            if(player_teleport(pmv)){
                player_process_old_pos(pmv);
                free(pmv);
                return PLAY_CONTINUE|PLAY_RECORD;
            }
            free(pmv);
            return PLAY_CONTINUE;
            break;
        default:
            break;
    }
    if((omv=player_process_move(pmv))){
        if(omv==pmv){
            game_win_move_player(pmv);
            if(cont&CT_PICKUP)
                player_process_collect(pmv);
            player_process_old_pos(pmv);
        }
        else{
            game_win_move_object(omv);
            game_win_move_player(pmv);
            pushed_process_new_pos(omv);
            player_process_old_pos(pmv);
        }
        free(pmv);
        return PLAY_PROCESS_MOVE|PLAY_RECORD;
    }
    free(pmv);
    return PLAY_CONTINUE;
}

struct xor_move* player_process_move(struct xor_move* pmv)
{
    struct xor_move* omv = pmv;
    enum CONTACT cont, ncont;
    cont=actions[pmv->to_obj].cont;
    #ifdef MOVE_DEBUG
    printf("player process move(xor_move* %lx)\n",(unsigned long)pmv);
    printf("cont:%d\n",cont);
    #endif
    if(cont!=CT_PASS){
        if (cont&CT_PUSH||cont&CT_HARDPUSH){
            omv=create_xor_move(pmv->to_x,pmv->to_y,pmv->dir);
            if((ncont=move_object_init(omv))!=CT_PASS){
                free(omv);
                return 0;
            }
            map->data[omv->from_y][omv->from_x]=ICON_FLOOR;
            if(cont&CT_FLIP)
                omv->from_obj=flip(omv->from_obj,omv->dir);
            map->data[omv->to_y][omv->to_x]=omv->from_obj;
        }
    }
    map->player[player.player].x=pmv->to_x;
    map->player[player.player].y=pmv->to_y;
    map->data[pmv->to_y][pmv->to_x]=pmv->from_obj;
    map->data[pmv->from_y][pmv->from_x]=ICON_FLOOR;
    return omv;
}


enum PLAY_STATE player_process_old_pos(struct xor_move* pmv)
{
    struct xor_move* gravmv;
    xy_t x,y;
    su_t gdir[3];
    su_t check[3];
    su_t n;
    #ifdef MOVE_DEBUG
    printf("\nplayer_process_old_pos(xor_move* pmv=%lx)\n",
        (unsigned long)pmv);
    #endif
    if(pmv->dir&MV_HORIZ){
        check[0]=gdir[1]=MV_UP;
        check[1]=gdir[0]=MV_DOWN;
        if(pmv->dir&MV_LEFT){
            check[2]=MV_RIGHT;
            gdir[2]=MV_LEFT;
        }
        else{
            check[2]=MV_LEFT;
            gdir[2]=MV_RIGHT;
        }
    }
    else{
        check[0]=gdir[1]=MV_RIGHT;
        check[1]=gdir[0]=MV_LEFT;
        if(pmv->dir&MV_DOWN){
            check[2]=MV_UP;
            gdir[2]=MV_DOWN;
        }
        else{
            check[2]=MV_DOWN;
            gdir[2]=MV_UP;
        }
    }
    for(n=0;n<3;n++){
        x=pmv->from_x;
        y=pmv->from_y;
        switch(check[n]){
            case MV_LEFT:   x-=1;   break;
            case MV_RIGHT:  x+=1;   break;
            case MV_UP:     y-=1;   break;
            case MV_DOWN:   y+=1;   break;
            default:
                break;
        }
        if((gravmv=create_gravity_chain_xydir(x,y,gdir[n]))){
            xmvlist_create();
            xmvlist_append_xor_move(gravmv);
            move_gravity_process();
        }
    }
    return PLAY_PROCESS_MOVE;
}

enum PLAY_STATE pushed_process_new_pos(struct xor_move* omv)
{
    omv->from_x=omv->to_x;
    omv->from_y=omv->to_y;
    if(actions[omv->from_obj].cont&CT_PUSH){
        #ifdef MOVE_DEBUG
        printf("pushed object & CT_PUSH\n");
        #endif
        /* although create_gravity_chain() detects the same two conditions  *
         * below, having them here also, creates the correct behaviour...   *
         * a bomb should only detonate if an object has gravitated onto it, *
         * not if an object was pushed onto it.                             */
        if(actions[omv->from_obj].mvini==MVI_GRAVITY){
            omv->dir=actions[omv->from_obj].mvi_dir;
            if(move_object_init(omv)==CT_PASS){
                if(create_gravity_chain(omv)){
                    xmvlist_create();
                    xmvlist_append_xor_move(omv);
                    move_gravity_process();
                    return PLAY_PROCESS_MOVE;
                }
            }
        }
        free(omv);
        return PLAY_PROCESS_MOVE;
    }
    if(actions[omv->from_obj].cont&CT_HARDPUSH){
        #ifdef MOVE_DEBUG
        printf("pushed object & CT_HARDPUSH\n");
        #endif
        move_hard_push(omv);
        if(actions[omv->from_obj].mvini==MVI_GRAVITY){
            omv->dir=actions[omv->from_obj].mvi_dir;
            if(move_object_init(omv)==CT_PASS){
                if(create_gravity_chain(omv)){
                    xmvlist_create();
                    xmvlist_append_xor_move(omv);
                    move_gravity_process();
                    return PLAY_PROCESS_MOVE;
                }
            }
        }
        free(omv);
        return PLAY_PROCESS_MOVE;
    }
    #ifdef MOVE_DEBUG
    printf("non gravitating or hard-pusha\n");
    #endif
    free(omv);
    return PLAY_PROCESS_MOVE;
}

/*
    if(move_object_init(omv)==CT_PASS){
        if(actions[omv->from_obj].cont==CT_HARDPUSH){
            move_hard_push(omv);
            omv->dir=actions[omv->from_obj].mvi_dir;
            if(move_object_init(omv)!=CT_PASS){
                free(omv);
                return PLAY_PROCESS_MOVE;
            }
        }
        if(create_gravity_chain(omv)){
            xmvlist_create();
            xmvlist_append_xor_move(omv);
            move_gravity_process();
        }
        else
            free(omv);
    }
    else if(actions[omv->from_obj].cont==CT_HARDPUSH
          &&actions[omv->from_obj].mvini==MVI_GRAVITY)
    {
        omv->dir=actions[omv->from_obj].mvi_dir;
        if(move_object_init(omv)!=CT_PASS){
            free(omv);
            return PLAY_PROCESS_MOVE;
        }
        if(create_gravity_chain(omv)){
            xmvlist_create();
            xmvlist_append_xor_move(omv);
            move_gravity_process();
        }
        else
            free(omv);
    }
    else
        free(omv);
    return PLAY_PROCESS_MOVE;
*/


void player_process_collect(struct xor_move* pmv)
{
    #ifdef MOVE_DEBUG
    printf("player process collect(xor_move* %lx)\n",(unsigned long)pmv);
    #endif
    switch(pmv->to_obj){
        case ICON_MASK:
            player.masks_collected++;
            if(!options->game)
                inc_max_moves();
            break;
        case ICON_MAP:
            player_process_map_pc(pmv);
            break;
        default:
            if(actions[pmv->to_obj].cont&CT_SWITCH){
                player.wall_vis=(player.wall_vis?FALSE:TRUE);
                init_wall(map->level,player.wall_vis);
                game_win_display();
            }
    }
}

void player_death(su_t icon)
{
    su_t p=(icon==ICON_PLAYER0)?0:1;
    char* msg[2]={" Oops! ","Gotcha!"};
    map->data[map->player[p].y][map->player[p].x]=ICON_FLOOR;
    if(p)
        player.p1_alive=0;
    else
        player.p0_alive=0;
    if(p==player.player){
        if((p?player.p0_alive:player.p1_alive))
            player.player^=1;
    }
    gfx_message(msg[p]);
    game_win_display();
}

void player_process_map_pc(struct xor_move* pmv)
{
    su_t i;
    su_t mb=1;
    for(i=0;i<4;i++,mb*=2){
        if(pmv->to_x==map->mappc[i].x
         &&pmv->to_y==map->mappc[i].y){
            player.have_map|=mb;
            #ifdef MAP_DEBUG
            printf("Map collected:(section %d bit)%d have_map:%d",i,mb,player.have_map);
            #endif
        }
    }
}
