/*
    XorGramana Copyright 2009 James W. Morris, james@jwm-art.net

    This file is part of XorGramana.

    XorGramana is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    XorGramana is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with XorGramana.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _PLAYER_H
#define _PLAYER_H

#include "actions.h"
#include "map.h"

enum PLAY_STATE
{
    PLAY_CONTINUE,          /* no movement, as such             */
    PLAY_PROCESS_MOVE,      /* process movement                 */
    PLAY_SWAP,              /* player swap                      */
    PLAY_QUIT,              /* user quit                        */
    PLAY_ZERO_MOVES,        /* all moves depleted               */
    PLAY_GOTCHA,            /* player death                     */
    PLAY_COMPLETE,          /* masks collected, exit reached    */
    PLAY_RECORD=0x80        /* record move to replay - or'd with above values if appropriate */
};

struct xor_player
{
    ctr_t move_no;           /* no longer decrementing from max_moves, due to mask pickup   *
                              * incrementing the noof moves available.                      */
    ctr_t masks_collected;
    unsigned player     : 1; /* set = player 2, not set = player 1 */
    unsigned p0_alive   : 1;
    unsigned p1_alive   : 1;
    unsigned wall_vis   : 1;
    unsigned replay     : 1;
    unsigned have_map   : 4; /* 1 bit set for each piece collected */
} player;

void player_init();


/*
    see map.h map.c for xor_map definition. see actions.h for
    values for move variable passed to player_move.
*/

/* player move, initiates player movement, but does not move player */
enum PLAY_STATE player_move(su_t move);

/* player_process_move

    moves player - and thing that player pushes - if any.
    return values:
        a) 0    if object player can push is blocked
        b) a new xor_move pointer
                if object can be pushed - pointer is the xor_move
                for object to be pushed.
        c) the same xor_move pointer as player's xor_move
                if there is nothing to push

*/
struct xor_move* player_process_move(struct xor_move* pmv);
enum PLAY_STATE player_process_old_pos(struct xor_move* pmv);
enum PLAY_STATE pushed_process_new_pos(struct xor_move* omv);

/* player_process_collect
    processes collection of objects by player,
    ie masks (and sad masks/switch), map-pieces.
    refers to the xor_move (rather than map) for
    identification of object.
*/
void player_process_collect(struct xor_move* pmv);
void player_death(su_t icon);
void player_process_map_pc(struct xor_move* pmv);

#endif
