/*
    XorGramana Copyright 2009 James W. Morris, james@jwm-art.net

    This file is part of XorGramana.

    XorGramana is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    XorGramana is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with XorGramana.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdlib.h>

#include "gfx.h"

#include "level_menu.h"

#include "movelist.h"

#include "play_xor.h"
#include "options.h"
#include "scores.h"
#include "input.h"
#include "mapdisplay.h"
#include "actions.h"

void splash();

int main(void)
{
    lvl_t lastlevel;
    lvl_t level=1;
    if(!options_create()){
        printf("Please either install XorGramana by running 'sudo make install' within the\n");
        printf("XorGramana source directory, or always start XorGramana from within that\n");
        printf("directory.\n");
        exit(1);
    }
    create_scores();
    load_scores();
    gfx_start();
    init_icons();
    icons_init_overlays();
    game_win_create();
    level_win_create();
    play_create_keys();
    map_display_create_keys();
    key_repeat_on();
    scr_win_border();
    do{
        lastlevel=level;
        level=level_menu(level);
        switch(level){
            case 100:
                if((level=replay_load())<0)
                    level=lastlevel;
                else if(level==100){
                    while(play_xor(level,FALSE));
                        level=replay.level;
                    level=replay.level;
                    scr_win_border();
                }
                break;
            case 999:
                level=lastlevel;
                help();
                break;
            default:
                if(level>=MIN_LEVEL&&level<=MAX_LEVEL){
                    if(play_xor(level,TRUE)){
                        while(play_xor(level,FALSE)){
                            scr_win_border();
                        }
                        level=replay.level;
                    }
                    scr_win_border();
                }
        }
    }while(level>=0&&!xginput.exit);
    map_display_destroy_keys();
    map_display_destroy_icons();
    play_destroy_keys();
    game_win_destroy();
    level_win_destroy();
    icons_cleanup();
    gfx_end();
    destroy_scores();
    options_destroy_map_names();
    options_destroy();
    return 0;
}

